@@ -109,7 +109,12 @@ public final class AccountingConstants {
         INCOME_FROM_CHARGE_OFF_PENALTY(18), //
         INCOME_FROM_GOODWILL_CREDIT_INTEREST(19), //
         INCOME_FROM_GOODWILL_CREDIT_FEES(20), //
-        INCOME_FROM_GOODWILL_CREDIT_PENALTY(21); //
+        INCOME_FROM_GOODWILL_CREDIT_PENALTY(21), //
+        INCOME_FROM_CAPITALIZATION(22), //
+        DEFERRED_INCOME_LIABILITY(23), //
+        BUY_DOWN_EXPENSE(24), //
+        INCOME_FROM_BUY_DOWN(25), //
+        ;
 
         private final Integer value;
 
@@ -175,7 +180,12 @@ public final class AccountingConstants {
         INCOME_FROM_GOODWILL_CREDIT_PENALTY("incomeFromGoodwillCreditPenaltyAccountId"), //
         CHARGE_OFF_REASON_TO_EXPENSE_ACCOUNT_MAPPINGS("chargeOffReasonToExpenseAccountMappings"), //
         EXPENSE_GL_ACCOUNT_ID("expenseAccountId"), //
-        CHARGE_OFF_REASON_CODE_VALUE_ID("chargeOffReasonCodeValueId"); //
+        CHARGE_OFF_REASON_CODE_VALUE_ID("chargeOffReasonCodeValueId"), //
+        DEFERRED_INCOME_LIABILITY("deferredIncomeLiabilityAccountId"), //
+        INCOME_FROM_CAPITALIZATION("incomeFromCapitalizationAccountId"), //
+        BUY_DOWN_EXPENSE("buyDownExpenseAccountId"), //
+        INCOME_FROM_BUY_DOWN("incomeFromBuyDownAccountId"), //
+        ;
 
         private final String value;
 
@@ -185,7 +195,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public String getValue() {
@@ -217,7 +227,12 @@ public final class AccountingConstants {
         INCOME_FROM_CHARGE_OFF_PENALTY("incomeFromChargeOffPenaltyAccount"), //
         INCOME_FROM_GOODWILL_CREDIT_INTEREST("incomeFromGoodwillCreditInterestAccount"), //
         INCOME_FROM_GOODWILL_CREDIT_FEES("incomeFromGoodwillCreditFeesAccount"), //
-        INCOME_FROM_GOODWILL_CREDIT_PENALTY("incomeFromGoodwillCreditPenaltyAccount"); //
+        INCOME_FROM_GOODWILL_CREDIT_PENALTY("incomeFromGoodwillCreditPenaltyAccount"), //
+        DEFERRED_INCOME_LIABILITY("deferredIncomeLiabilityAccount"), //
+        INCOME_FROM_CAPITALIZATION("incomeFromCapitalizationAccount"), //
+        BUY_DOWN_EXPENSE("buyDownExpenseAccount"), //
+        INCOME_FROM_BUY_DOWN("incomeFromBuyDownAccount"), //
+        ;
 
         private final String value;
 
@@ -227,7 +242,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public String getValue() {
@@ -259,7 +274,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public Integer getValue() {
@@ -307,7 +322,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public Integer getValue() {
@@ -362,7 +377,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public String getValue() {
@@ -399,7 +414,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public String getValue() {
@@ -439,7 +454,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public Integer getValue() {
@@ -491,7 +506,10 @@ public final class AccountingConstants {
      ***/
     public enum CashAccountsForShares {
 
-        SHARES_REFERENCE(1), SHARES_SUSPENSE(2), INCOME_FROM_FEES(3), SHARES_EQUITY(4);
+        SHARES_REFERENCE(1), //
+        SHARES_SUSPENSE(2), //
+        INCOME_FROM_FEES(3), //
+        SHARES_EQUITY(4); //
 
         private final Integer value;
 
@@ -501,7 +519,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public Integer getValue() {
@@ -540,7 +558,7 @@ public final class AccountingConstants {
 
         @Override
         public String toString() {
-            return name().toString().replaceAll("_", " ");
+            return name().replaceAll("_", " ");
         }
 
         public String getValue() {
