@@ -23,18 +23,24 @@ import static java.time.temporal.ChronoUnit.DAYS;
 import jakarta.validation.constraints.NotNull;
 import java.time.LocalDate;
 import java.time.LocalDateTime;
+import java.time.LocalTime;
 import java.time.OffsetDateTime;
 import java.time.ZoneId;
 import java.time.ZoneOffset;
 import java.time.format.DateTimeFormatter;
+import java.time.format.DateTimeFormatterBuilder;
 import java.time.format.DateTimeParseException;
+import java.time.format.ResolverStyle;
+import java.time.temporal.ChronoField;
 import java.time.temporal.ChronoUnit;
+import java.time.temporal.TemporalAccessor;
 import java.util.List;
 import java.util.Locale;
 import java.util.Optional;
 import org.apache.fineract.infrastructure.core.data.ApiParameterError;
 import org.apache.fineract.infrastructure.core.domain.FineractPlatformTenant;
 import org.apache.fineract.infrastructure.core.exception.PlatformApiDataValidationException;
+import org.apache.fineract.infrastructure.core.serialization.JsonParserHelper;
 
 public final class DateUtils {
 
@@ -124,18 +130,10 @@ public final class DateUtils {
         return compare(first, second, truncate) == 0;
     }
 
-    public static boolean isEqualTenantDateTime(LocalDateTime dateTime) {
-        return isEqualTenantDateTime(dateTime, null);
-    }
-
     public static boolean isEqualTenantDateTime(LocalDateTime dateTime, ChronoUnit truncate) {
         return isEqual(dateTime, getLocalDateTimeOfTenant(), truncate);
     }
 
-    public static boolean isEqualSystemDateTime(LocalDateTime dateTime) {
-        return isEqualSystemDateTime(dateTime, null);
-    }
-
     public static boolean isEqualSystemDateTime(LocalDateTime dateTime, ChronoUnit truncate) {
         return isEqual(dateTime, getLocalDateTimeOfSystem(), truncate);
     }
@@ -148,22 +146,6 @@ public final class DateUtils {
         return compare(first, second, truncate) < 0;
     }
 
-    public static boolean isBeforeTenantDateTime(LocalDateTime dateTime) {
-        return isBeforeTenantDateTime(dateTime, null);
-    }
-
-    public static boolean isBeforeTenantDateTime(LocalDateTime dateTime, ChronoUnit truncate) {
-        return isBefore(dateTime, getLocalDateTimeOfTenant(), truncate);
-    }
-
-    public static boolean isBeforeSystemDateTime(LocalDateTime dateTime) {
-        return isBeforeSystemDateTime(dateTime, null);
-    }
-
-    public static boolean isBeforeSystemDateTime(LocalDateTime dateTime, ChronoUnit truncate) {
-        return isBefore(dateTime, getLocalDateTimeOfSystem(), truncate);
-    }
-
     public static boolean isAfter(LocalDateTime first, LocalDateTime second) {
         return isAfter(first, second, null);
     }
@@ -180,10 +162,6 @@ public final class DateUtils {
         return isAfter(dateTime, getLocalDateTimeOfTenant(), truncate);
     }
 
-    public static boolean isAfterSystemDateTime(LocalDateTime dateTime) {
-        return isAfterSystemDateTime(dateTime, null);
-    }
-
     public static boolean isAfterSystemDateTime(LocalDateTime dateTime, ChronoUnit truncate) {
         return isAfter(dateTime, getLocalDateTimeOfSystem(), truncate);
     }
@@ -224,10 +202,6 @@ public final class DateUtils {
         return compare(first, second, truncate) == 0;
     }
 
-    public static boolean isEqualTenantDateTime(OffsetDateTime dateTime) {
-        return isEqualTenantDateTime(dateTime, null);
-    }
-
     public static boolean isEqualTenantDateTime(OffsetDateTime dateTime, ChronoUnit truncate) {
         return isEqual(dateTime, getOffsetDateTimeOfTenant(), truncate);
     }
@@ -240,10 +214,6 @@ public final class DateUtils {
         return compare(first, second, truncate) < 0;
     }
 
-    public static boolean isBeforeTenantDateTime(OffsetDateTime dateTime) {
-        return isBeforeTenantDateTime(dateTime, null);
-    }
-
     public static boolean isBeforeTenantDateTime(OffsetDateTime dateTime, ChronoUnit truncate) {
         return isBefore(dateTime, getOffsetDateTimeOfTenant(), truncate);
     }
@@ -256,10 +226,6 @@ public final class DateUtils {
         return compare(first, second, truncate) > 0;
     }
 
-    public static boolean isAfterTenantDateTime(OffsetDateTime dateTime) {
-        return isAfterTenantDateTime(dateTime, null);
-    }
-
     public static boolean isAfterTenantDateTime(OffsetDateTime dateTime, ChronoUnit truncate) {
         return isAfter(dateTime, getOffsetDateTimeOfTenant(), truncate);
     }
@@ -270,10 +236,6 @@ public final class DateUtils {
         return ThreadLocalContextUtil.getBusinessDate();
     }
 
-    public static int compareToBusinessDate(LocalDate date) {
-        return compare(date, getBusinessLocalDate());
-    }
-
     public static boolean isEqualTenantDate(LocalDate date) {
         return isEqual(date, getLocalDateOfTenant());
     }
@@ -282,10 +244,6 @@ public final class DateUtils {
         return isBefore(date, getLocalDateOfTenant());
     }
 
-    public static boolean isAfterTenantDate(LocalDate date) {
-        return isAfter(date, getLocalDateOfTenant());
-    }
-
     public static boolean isEqualBusinessDate(LocalDate date) {
         return isEqual(date, getBusinessLocalDate());
     }
@@ -302,6 +260,12 @@ public final class DateUtils {
         return isAfterBusinessDate(localDate);
     }
 
+    public static boolean isDateInRangeFromInclusiveToExclusive(final LocalDate fromInclusive, final LocalDate upToNotInclusive,
+            final LocalDate target) {
+        return (DateUtils.isEqual(target, fromInclusive) || DateUtils.isAfter(target, fromInclusive))
+                && DateUtils.isBefore(target, upToNotInclusive);
+    }
+
     public static int compare(LocalDate first, LocalDate second) {
         return compare(first, second, true);
     }
@@ -334,6 +298,10 @@ public final class DateUtils {
         return first != null && (second == null || first.isAfter(second));
     }
 
+    public static boolean isAfterInclusive(LocalDate first, LocalDate second) {
+        return isAfter(first, second) || isEqual(first, second);
+    }
+
     public static long getDifference(LocalDate first, LocalDate second, @NotNull ChronoUnit unit) {
         if (first == null || second == null) {
             throw new IllegalArgumentException("Dates must not be null to get difference");
@@ -381,6 +349,16 @@ public final class DateUtils {
         }
     }
 
+    public static LocalDate toLocalDate(String local, String date, String dateFormat) {
+        Locale locale = Locale.forLanguageTag(local);
+        String patternISO = dateFormat.replace("y", "u");
+        DateTimeFormatter formatter = new DateTimeFormatterBuilder().parseCaseInsensitive().parseLenient().appendPattern(patternISO)
+                .optionalStart().appendPattern(" HH:mm:ss").optionalEnd().parseDefaulting(ChronoField.HOUR_OF_DAY, 0)
+                .parseDefaulting(ChronoField.MINUTE_OF_HOUR, 0).parseDefaulting(ChronoField.SECOND_OF_MINUTE, 0).toFormatter(locale)
+                .withResolverStyle(ResolverStyle.STRICT);
+        return LocalDateTime.parse(date, formatter).toLocalDate();
+    }
+
     public static String format(LocalDate date) {
         return format(date, null);
     }
@@ -458,4 +436,29 @@ public final class DateUtils {
         }
         return formatter;
     }
+
+    public static LocalDateTime convertDateTimeStringToLocalDateTime(String dateTimeStr, String dateFormat, String localeStr,
+            LocalTime fallbackTime) {
+        if (dateTimeStr == null || dateTimeStr.isBlank()) {
+            return null;
+        }
+        final Locale locale = localeStr == null ? null : JsonParserHelper.localeFromString(localeStr);
+        DateTimeFormatter formatter = getDateFormatter(dateFormat, locale);
+        TemporalAccessor parsed = formatter.parse(dateTimeStr);
+
+        boolean hasTime = parsed.isSupported(ChronoField.HOUR_OF_DAY) && parsed.isSupported(ChronoField.MINUTE_OF_HOUR);
+
+        try {
+            if (hasTime) {
+                return LocalDateTime.from(parsed);
+            } else {
+                LocalDate date = LocalDate.from(parsed);
+                return LocalDateTime.of(date, fallbackTime);
+            }
+        } catch (final DateTimeParseException e) {
+            final List<ApiParameterError> errors = List.of(ApiParameterError.parameterError("validation.msg.invalid.date.pattern",
+                    "The parameter date (" + dateTimeStr + ") format is invalid", "date", dateTimeStr));
+            throw new PlatformApiDataValidationException("validation.msg.validation.errors.exist", "Validation errors exist.", errors, e);
+        }
+    }
 }
