@@ -31,6 +31,7 @@ import java.util.Map;
 import java.util.Set;
 import lombok.Getter;
 import org.apache.fineract.infrastructure.core.domain.LocalDateInterval;
+import org.apache.fineract.infrastructure.core.jersey.serializer.legacy.JsonLocalDateArrayFormat;
 import org.apache.fineract.infrastructure.core.service.DateUtils;
 import org.apache.fineract.organisation.monetary.data.CurrencyData;
 import org.apache.fineract.organisation.monetary.domain.MonetaryCurrency;
@@ -51,6 +52,7 @@ import org.springframework.util.CollectionUtils;
  * Immutable data object representing a savings account transaction.
  */
 @Getter
+@JsonLocalDateArrayFormat
 public final class SavingsAccountTransactionData implements Serializable {
 
     private Long id;
@@ -70,7 +72,7 @@ public final class SavingsAccountTransactionData implements Serializable {
     private final LocalDate submittedOnDate;
     private final boolean interestedPostedAsOn;
     private final String submittedByUsername;
-    private final String note;
+    private String note;
     private final boolean isManualTransaction;
     private final Boolean isReversal;
     private final Long originalTransactionId;
@@ -253,8 +255,8 @@ public final class SavingsAccountTransactionData implements Serializable {
     }
 
     public static SavingsAccountTransactionData importInstance(BigDecimal transactionAmount, LocalDate transactionDate, Long paymentTypeId,
-            String accountNumber, String checkNumber, String routingCode, String receiptNumber, String bankNumber, Long savingsAccountId,
-            SavingsAccountTransactionEnumData transactionType, Integer rowIndex, String locale, String dateFormat) {
+            String accountNumber, String checkNumber, String routingCode, String receiptNumber, String bankNumber, String note,
+            Long savingsAccountId, SavingsAccountTransactionEnumData transactionType, Integer rowIndex, String locale, String dateFormat) {
         SavingsAccountTransactionData data = createImport(transactionType, null, savingsAccountId, accountNumber, transactionDate,
                 transactionAmount, false, transactionDate, false, false);
         data.rowIndex = rowIndex;
@@ -263,6 +265,7 @@ public final class SavingsAccountTransactionData implements Serializable {
         data.routingCode = routingCode;
         data.receiptNumber = receiptNumber;
         data.bankNumber = bankNumber;
+        data.note = note;
         data.locale = locale;
         data.dateFormat = dateFormat;
         return data;
