@@ -18,8 +18,12 @@
  */
 package org.apache.fineract.investor.service;
 
+import static org.apache.fineract.investor.data.ExternalTransferStatus.ACTIVE_INTERMEDIATE;
 import static org.apache.fineract.investor.data.ExternalTransferStatus.PENDING;
+import static org.apache.fineract.investor.data.ExternalTransferStatus.PENDING_INTERMEDIATE;
 import static org.apache.fineract.portfolio.loanaccount.domain.LoanStatus.ACTIVE;
+import static org.apache.fineract.portfolio.loanaccount.domain.LoanStatus.CLOSED_OBLIGATIONS_MET;
+import static org.apache.fineract.portfolio.loanaccount.domain.LoanStatus.OVERPAID;
 import static org.apache.fineract.portfolio.loanaccount.domain.LoanStatus.TRANSFER_IN_PROGRESS;
 import static org.apache.fineract.portfolio.loanaccount.domain.LoanStatus.TRANSFER_ON_HOLD;
 
@@ -35,9 +39,10 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
+import java.util.stream.Collectors;
 import lombok.RequiredArgsConstructor;
 import org.apache.commons.lang3.StringUtils;
-import org.apache.fineract.cob.data.LoanIdAndExternalIdAndStatus;
+import org.apache.fineract.cob.data.LoanDataForExternalTransfer;
 import org.apache.fineract.infrastructure.core.api.JsonCommand;
 import org.apache.fineract.infrastructure.core.data.ApiParameterError;
 import org.apache.fineract.infrastructure.core.data.CommandProcessingResult;
@@ -70,26 +75,55 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
 
     private static final LocalDate FUTURE_DATE_9999_12_31 = LocalDate.of(9999, 12, 31);
     private static final List<LoanStatus> ACTIVE_LOAN_STATUSES = List.of(ACTIVE, TRANSFER_IN_PROGRESS, TRANSFER_ON_HOLD);
+    private static final List<LoanStatus> VALID_DELAYED_SETTLEMENT_LOAN_STATUSES_BUYBACK_AND_SALE = List.of(ACTIVE, TRANSFER_IN_PROGRESS,
+            TRANSFER_ON_HOLD, OVERPAID, CLOSED_OBLIGATIONS_MET);
     private static final List<ExternalTransferStatus> BUYBACK_READY_STATUSES = List.of(ExternalTransferStatus.PENDING,
             ExternalTransferStatus.ACTIVE);
+    private static final List<ExternalTransferStatus> BUYBACK_READY_STATUSES_FOR_DELAY_SETTLEMENT = List
+            .of(ExternalTransferStatus.ACTIVE_INTERMEDIATE, ExternalTransferStatus.ACTIVE);
     private final ExternalAssetOwnerTransferRepository externalAssetOwnerTransferRepository;
     private final ExternalAssetOwnerRepository externalAssetOwnerRepository;
     private final FromJsonHelper fromApiJsonHelper;
     private final LoanRepository loanRepository;
+    private final DelayedSettlementAttributeService delayedSettlementAttributeService;
+
+    @Override
+    @Transactional
+    public CommandProcessingResult intermediarySaleLoanByLoanId(JsonCommand command) {
+        final JsonElement json = fromApiJsonHelper.parse(command.json());
+        validateIntermediarySaleRequestBody(command.json());
+        Long loanId = command.getLoanId();
+        LoanDataForExternalTransfer loanDataForExternalTransfer = fetchAndValidateLoanDataForExternalTransfer(loanId);
+        if (!delayedSettlementAttributeService.isEnabled(loanDataForExternalTransfer.getLoanProductId())) {
+            throw new ExternalAssetOwnerInitiateTransferException(
+                    String.format("Delayed Settlement Configuration is not enabled for the loan product: %s",
+                            loanDataForExternalTransfer.getLoanProductShortName()));
+        }
+        ExternalId externalId = getTransferExternalIdFromJson(json);
+        validateExternalId(externalId);
+        validateLoanStatusIntermediarySale(loanDataForExternalTransfer);
+        ExternalAssetOwnerTransfer intermediarySaleTransfer = createIntermediarySaleTransfer(loanId, json,
+                loanDataForExternalTransfer.getExternalId());
+        validateIntermediarySale(intermediarySaleTransfer);
+        externalAssetOwnerTransferRepository.saveAndFlush(intermediarySaleTransfer);
+        return buildResponseData(intermediarySaleTransfer);
+    }
 
     @Override
     @Transactional
     public CommandProcessingResult saleLoanByLoanId(JsonCommand command) {
         final JsonElement json = fromApiJsonHelper.parse(command.json());
+        final LoanDataForExternalTransfer loanDataForExternalTransfer = fetchAndValidateLoanDataForExternalTransfer(command.getLoanId());
+        final boolean isDelayedSettlementEnabled = delayedSettlementAttributeService
+                .isEnabled(loanDataForExternalTransfer.getLoanProductId());
         validateSaleRequestBody(command.json());
         ExternalId externalId = getTransferExternalIdFromJson(json);
         validateExternalId(externalId);
         Long loanId = command.getLoanId();
-        LoanIdAndExternalIdAndStatus loanIdAndExternalIdAndStatus = fetchLoanDetails(loanId);
-        validateLoanStatus(loanIdAndExternalIdAndStatus);
+        validateLoanStatus(loanDataForExternalTransfer, isDelayedSettlementEnabled);
         ExternalAssetOwnerTransfer externalAssetOwnerTransfer = createSaleTransfer(loanId, json,
-                loanIdAndExternalIdAndStatus.getExternalId());
-        validateSale(externalAssetOwnerTransfer);
+                loanDataForExternalTransfer.getExternalId());
+        validateSale(externalAssetOwnerTransfer, isDelayedSettlementEnabled);
         externalAssetOwnerTransferRepository.saveAndFlush(externalAssetOwnerTransfer);
         return buildResponseData(externalAssetOwnerTransfer);
     }
@@ -99,13 +133,13 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
     public CommandProcessingResult buybackLoanByLoanId(JsonCommand command) {
         final JsonElement json = fromApiJsonHelper.parse(command.json());
         validateBuybackRequestBody(command.json());
-        Long loanId = command.getLoanId();
-        validateLoan(loanId);
+        LoanDataForExternalTransfer loanDataForExternalTransfer = fetchAndValidateLoanDataForExternalTransfer(command.getLoanId());
         LocalDate settlementDate = getSettlementDateFromJson(json);
         ExternalId externalId = getTransferExternalIdFromJson(json);
         validateSettlementDate(settlementDate);
         validateExternalId(externalId);
-        ExternalAssetOwnerTransfer effectiveTransfer = fetchAndValidateEffectiveTransferForBuyback(loanId, settlementDate);
+        ExternalAssetOwnerTransfer effectiveTransfer = fetchAndValidateEffectiveTransferForBuyback(loanDataForExternalTransfer,
+                settlementDate);
         ExternalAssetOwnerTransfer externalAssetOwnerTransfer = createBuybackTransfer(effectiveTransfer, settlementDate, externalId);
         externalAssetOwnerTransferRepository.saveAndFlush(externalAssetOwnerTransfer);
         return buildResponseData(externalAssetOwnerTransfer);
@@ -120,10 +154,8 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         }
     }
 
-    private void validateLoan(Long loanId) {
-        if (!loanRepository.existsById(loanId)) {
-            throw new LoanNotFoundException(loanId);
-        }
+    private LoanDataForExternalTransfer fetchAndValidateLoanDataForExternalTransfer(Long loanId) {
+        return loanRepository.findLoanDataForExternalTransferByLoanId(loanId).orElseThrow(() -> new LoanNotFoundException(loanId));
     }
 
     @Override
@@ -136,10 +168,7 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         return buildResponseData(cancelTransfer);
     }
 
-    private void validateEffectiveTransferForSale(final ExternalAssetOwnerTransfer externalAssetOwnerTransfer) {
-        List<ExternalAssetOwnerTransfer> effectiveTransfers = externalAssetOwnerTransferRepository
-                .findEffectiveTransfersOrderByIdDesc(externalAssetOwnerTransfer.getLoanId(), DateUtils.getBusinessLocalDate());
-
+    private void validateEffectiveTransferForSale(final List<ExternalAssetOwnerTransfer> effectiveTransfers) {
         if (effectiveTransfers.size() == 2) {
             throw new ExternalAssetOwnerInitiateTransferException("This loan cannot be sold, there is already an in progress transfer");
         } else if (effectiveTransfers.size() == 1) {
@@ -156,9 +185,47 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         }
     }
 
-    private ExternalAssetOwnerTransfer fetchAndValidateEffectiveTransferForBuyback(final Long loanId, final LocalDate settlementDate) {
+    private void validateEffectiveTransferForDelayedSettlementSale(final List<ExternalAssetOwnerTransfer> effectiveTransfers) {
+        if (effectiveTransfers.size() > 1) {
+            throw new ExternalAssetOwnerInitiateTransferException("This loan cannot be sold, there is already an in progress transfer");
+        } else if (effectiveTransfers.size() == 1) {
+            if (!ACTIVE_INTERMEDIATE.equals(effectiveTransfers.get(0).getStatus())) {
+                throw new ExternalAssetOwnerInitiateTransferException(
+                        "This loan cannot be sold, because it is not in ACTIVE-INTERMEDIATE state.");
+            }
+        } else {
+            throw new ExternalAssetOwnerInitiateTransferException("This loan cannot be sold, no effective transfer found.");
+        }
+    }
+
+    private void validateEffectiveTransferForIntermediarySale(final ExternalAssetOwnerTransfer externalAssetOwnerTransfer) {
         List<ExternalAssetOwnerTransfer> effectiveTransfers = externalAssetOwnerTransferRepository
-                .findEffectiveTransfersOrderByIdDesc(loanId, DateUtils.getBusinessLocalDate());
+                .findEffectiveTransfersOrderByIdDesc(externalAssetOwnerTransfer.getLoanId(), DateUtils.getBusinessLocalDate());
+
+        if (effectiveTransfers.size() > 1) {
+            throw new ExternalAssetOwnerInitiateTransferException("This loan cannot be sold, there is already an in progress transfer");
+        } else if (effectiveTransfers.size() == 1) {
+            if (PENDING_INTERMEDIATE.equals(effectiveTransfers.get(0).getStatus())) {
+                throw new ExternalAssetOwnerInitiateTransferException(
+                        "External asset owner transfer is already in PENDING_INTERMEDIATE state for this loan");
+            } else if (ExternalTransferStatus.ACTIVE.equals(effectiveTransfers.get(0).getStatus())) {
+                throw new ExternalAssetOwnerInitiateTransferException(
+                        "This loan cannot be sold, because it is owned by an external asset owner");
+            } else {
+                throw new ExternalAssetOwnerInitiateTransferException(String.format(
+                        "This loan cannot be sold, because it is incorrect state! (transferId = %s)", effectiveTransfers.get(0).getId()));
+            }
+        }
+    }
+
+    private ExternalAssetOwnerTransfer fetchAndValidateEffectiveTransferForBuyback(
+            final LoanDataForExternalTransfer loanDataForExternalTransfer, final LocalDate settlementDate) {
+        if (delayedSettlementAttributeService.isEnabled(loanDataForExternalTransfer.getLoanProductId())) {
+            return fetchAndValidateEffectiveTransferForBuybackWithDelayedSettlement(loanDataForExternalTransfer, settlementDate);
+        }
+
+        List<ExternalAssetOwnerTransfer> effectiveTransfers = externalAssetOwnerTransferRepository
+                .findEffectiveTransfersOrderByIdDesc(loanDataForExternalTransfer.getId(), DateUtils.getBusinessLocalDate());
 
         if (effectiveTransfers.size() == 0) {
             throw new ExternalAssetOwnerInitiateTransferException(
@@ -179,6 +246,39 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         return effectiveTransfers.get(0);
     }
 
+    private ExternalAssetOwnerTransfer fetchAndValidateEffectiveTransferForBuybackWithDelayedSettlement(
+            final LoanDataForExternalTransfer loanDataForExternalTransfer, final LocalDate settlementDate) {
+        List<ExternalAssetOwnerTransfer> effectiveTransfers = externalAssetOwnerTransferRepository
+                .findEffectiveTransfersOrderByIdDesc(loanDataForExternalTransfer.getId(), DateUtils.getBusinessLocalDate());
+
+        if (effectiveTransfers.isEmpty()) {
+            throw new ExternalAssetOwnerInitiateTransferException(
+                    "This loan cannot be bought back, it is not owned by an external asset owner");
+        }
+
+        Set<ExternalTransferStatus> effectiveTransferStatuses = effectiveTransfers.stream().map(ExternalAssetOwnerTransfer::getStatus)
+                .collect(Collectors.toSet());
+
+        if (Set.of(ExternalTransferStatus.ACTIVE_INTERMEDIATE, ExternalTransferStatus.PENDING).equals(effectiveTransferStatuses)) {
+            throw new ExternalAssetOwnerInitiateTransferException("This loan cannot be bought back, external asset owner sale is pending");
+        } else if (Set.of(ExternalTransferStatus.ACTIVE_INTERMEDIATE, ExternalTransferStatus.BUYBACK_INTERMEDIATE)
+                .equals(effectiveTransferStatuses)
+                || Set.of(ExternalTransferStatus.ACTIVE, ExternalTransferStatus.BUYBACK).equals(effectiveTransferStatuses)) {
+            throw new ExternalAssetOwnerInitiateTransferException(
+                    "This loan cannot be bought back, external asset owner buyback transfer is already in progress");
+        } else if (!BUYBACK_READY_STATUSES_FOR_DELAY_SETTLEMENT.contains(effectiveTransfers.get(0).getStatus())) {
+            throw new ExternalAssetOwnerInitiateTransferException(
+                    String.format("This loan cannot be bought back, effective transfer is not in right state: %s",
+                            effectiveTransfers.get(0).getStatus()));
+        } else if (DateUtils.isBefore(settlementDate, effectiveTransfers.get(0).getSettlementDate())) {
+            throw new ExternalAssetOwnerInitiateTransferException(
+                    String.format("This loan cannot be bought back, settlement date is earlier than effective transfer settlement date: %s",
+                            effectiveTransfers.get(0).getSettlementDate()));
+        }
+
+        return effectiveTransfers.get(0);
+    }
+
     private ExternalAssetOwnerTransfer fetchAndValidateEffectiveTransferForCancel(final Long transferId) {
         ExternalAssetOwnerTransfer selectedTransfer = externalAssetOwnerTransferRepository.findById(transferId)
                 .orElseThrow(() -> new ExternalAssetOwnerInitiateTransferException(
@@ -206,10 +306,9 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         ExternalAssetOwnerTransfer externalAssetOwnerTransfer = new ExternalAssetOwnerTransfer();
         externalAssetOwnerTransfer.setExternalId(externalId);
         externalAssetOwnerTransfer.setOwner(effectiveTransfer.getOwner());
-        externalAssetOwnerTransfer.setStatus(ExternalTransferStatus.BUYBACK);
+        externalAssetOwnerTransfer.setStatus(determineStatusAfterBuyback(effectiveTransfer));
         externalAssetOwnerTransfer.setLoanId(effectiveTransfer.getLoanId());
         externalAssetOwnerTransfer.setExternalLoanId(effectiveTransfer.getExternalLoanId());
-        externalAssetOwnerTransfer.setOwner(effectiveTransfer.getOwner());
         externalAssetOwnerTransfer.setSettlementDate(settlementDate);
         externalAssetOwnerTransfer.setEffectiveDateFrom(effectiveDateFrom);
         externalAssetOwnerTransfer.setEffectiveDateTo(FUTURE_DATE_9999_12_31);
@@ -217,6 +316,16 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         return externalAssetOwnerTransfer;
     }
 
+    private ExternalTransferStatus determineStatusAfterBuyback(ExternalAssetOwnerTransfer effectiveTransfer) {
+        return switch (effectiveTransfer.getStatus()) {
+            case PENDING -> ExternalTransferStatus.BUYBACK;
+            case ACTIVE -> ExternalTransferStatus.BUYBACK;
+            case ACTIVE_INTERMEDIATE -> ExternalTransferStatus.BUYBACK_INTERMEDIATE;
+            default -> throw new ExternalAssetOwnerInitiateTransferException(String.format(
+                    "This loan cannot be bought back, effective transfer is not in right state: %s", effectiveTransfer.getStatus()));
+        };
+    }
+
     private ExternalAssetOwnerTransfer createCancelTransfer(ExternalAssetOwnerTransfer effectiveTransfer) {
         ExternalAssetOwnerTransfer externalAssetOwnerTransfer = new ExternalAssetOwnerTransfer();
         externalAssetOwnerTransfer.setExternalId(effectiveTransfer.getExternalId());
@@ -224,6 +333,7 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         externalAssetOwnerTransfer.setSubStatus(ExternalTransferSubStatus.USER_REQUESTED);
         externalAssetOwnerTransfer.setLoanId(effectiveTransfer.getLoanId());
         externalAssetOwnerTransfer.setExternalLoanId(effectiveTransfer.getExternalLoanId());
+        externalAssetOwnerTransfer.setExternalGroupId(effectiveTransfer.getExternalGroupId());
         externalAssetOwnerTransfer.setOwner(effectiveTransfer.getOwner());
         externalAssetOwnerTransfer.setSettlementDate(effectiveTransfer.getSettlementDate());
         externalAssetOwnerTransfer.setEffectiveDateFrom(effectiveTransfer.getEffectiveDateFrom());
@@ -241,9 +351,22 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
                 .build();
     }
 
-    private void validateSale(ExternalAssetOwnerTransfer externalAssetOwnerTransfer) {
+    private void validateSale(ExternalAssetOwnerTransfer externalAssetOwnerTransfer, boolean isDelayedSettlementEnabled) {
         validateSettlementDate(externalAssetOwnerTransfer);
-        validateEffectiveTransferForSale(externalAssetOwnerTransfer);
+
+        final List<ExternalAssetOwnerTransfer> effectiveTransfers = externalAssetOwnerTransferRepository
+                .findEffectiveTransfersOrderByIdDesc(externalAssetOwnerTransfer.getLoanId(), DateUtils.getBusinessLocalDate());
+
+        if (isDelayedSettlementEnabled) {
+            validateEffectiveTransferForDelayedSettlementSale(effectiveTransfers);
+        } else {
+            validateEffectiveTransferForSale(effectiveTransfers);
+        }
+    }
+
+    private void validateIntermediarySale(ExternalAssetOwnerTransfer externalAssetOwnerTransfer) {
+        validateSettlementDate(externalAssetOwnerTransfer);
+        validateEffectiveTransferForIntermediarySale(externalAssetOwnerTransfer);
     }
 
     private void validateSettlementDate(ExternalAssetOwnerTransfer externalAssetOwnerTransfer) {
@@ -256,9 +379,25 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         }
     }
 
-    private void validateLoanStatus(LoanIdAndExternalIdAndStatus entity) {
-        if (!ACTIVE_LOAN_STATUSES.contains(LoanStatus.fromInt(entity.getLoanStatus()))) {
-            throw new ExternalAssetOwnerInitiateTransferException("Loan is not in active status");
+    private void validateLoanStatus(LoanDataForExternalTransfer loanDataForExternalTransfer, boolean isDelayedSettlementEnabled) {
+        LoanStatus loanStatus = loanDataForExternalTransfer.getLoanStatus();
+        if (!getValidLoanStatusList(isDelayedSettlementEnabled).contains(loanStatus)) {
+            throw new ExternalAssetOwnerInitiateTransferException(String.format("Loan status %s is not valid for transfer.", loanStatus));
+        }
+    }
+
+    private void validateLoanStatusIntermediarySale(LoanDataForExternalTransfer loanDataForExternalTransfer) {
+        LoanStatus loanStatus = loanDataForExternalTransfer.getLoanStatus();
+        if (!ACTIVE_LOAN_STATUSES.contains(loanStatus)) {
+            throw new ExternalAssetOwnerInitiateTransferException(String.format("Loan status %s is not valid for transfer.", loanStatus));
+        }
+    }
+
+    private List<LoanStatus> getValidLoanStatusList(boolean isDelayedSettlementEnabled) {
+        if (isDelayedSettlementEnabled) {
+            return VALID_DELAYED_SETTLEMENT_LOAN_STATUSES_BUYBACK_AND_SALE;
+        } else {
+            return ACTIVE_LOAN_STATUSES;
         }
     }
 
@@ -276,14 +415,33 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         externalAssetOwnerTransfer.setEffectiveDateTo(FUTURE_DATE_9999_12_31);
         externalAssetOwnerTransfer.setLoanId(loanId);
         externalAssetOwnerTransfer.setExternalLoanId(externalLoanId);
+        externalAssetOwnerTransfer.setExternalGroupId(getTransferExternalGroupIdFromJson(json));
+        return externalAssetOwnerTransfer;
+    }
+
+    private ExternalAssetOwnerTransfer createIntermediarySaleTransfer(Long loanId, JsonElement json, ExternalId externalLoanId) {
+        ExternalAssetOwnerTransfer externalAssetOwnerTransfer = new ExternalAssetOwnerTransfer();
+        LocalDate effectiveFrom = ThreadLocalContextUtil.getBusinessDate();
+
+        ExternalAssetOwner owner = getOwner(json);
+        externalAssetOwnerTransfer.setOwner(owner);
+        externalAssetOwnerTransfer.setExternalId(getTransferExternalIdFromJson(json));
+        externalAssetOwnerTransfer.setStatus(PENDING_INTERMEDIATE);
+        externalAssetOwnerTransfer.setPurchasePriceRatio(getPurchasePriceRatioFromJson(json));
+        externalAssetOwnerTransfer.setSettlementDate(getSettlementDateFromJson(json));
+        externalAssetOwnerTransfer.setEffectiveDateFrom(effectiveFrom);
+        externalAssetOwnerTransfer.setEffectiveDateTo(FUTURE_DATE_9999_12_31);
+        externalAssetOwnerTransfer.setLoanId(loanId);
+        externalAssetOwnerTransfer.setExternalLoanId(externalLoanId);
+        externalAssetOwnerTransfer.setExternalGroupId(getTransferExternalGroupIdFromJson(json));
         return externalAssetOwnerTransfer;
     }
 
     private void validateSaleRequestBody(String apiRequestBodyAsJson) {
-        final Set<String> requestParameters = new HashSet<>(
-                Arrays.asList(ExternalTransferRequestParameters.SETTLEMENT_DATE, ExternalTransferRequestParameters.OWNER_EXTERNAL_ID,
-                        ExternalTransferRequestParameters.TRANSFER_EXTERNAL_ID, ExternalTransferRequestParameters.PURCHASE_PRICE_RATIO,
-                        ExternalTransferRequestParameters.DATEFORMAT, ExternalTransferRequestParameters.LOCALE));
+        final Set<String> requestParameters = new HashSet<>(Arrays.asList(ExternalTransferRequestParameters.SETTLEMENT_DATE,
+                ExternalTransferRequestParameters.OWNER_EXTERNAL_ID, ExternalTransferRequestParameters.TRANSFER_EXTERNAL_ID,
+                ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID, ExternalTransferRequestParameters.PURCHASE_PRICE_RATIO,
+                ExternalTransferRequestParameters.DATEFORMAT, ExternalTransferRequestParameters.LOCALE));
         final Type typeOfMap = new TypeToken<Map<String, Object>>() {
 
         }.getType();
@@ -308,6 +466,51 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         LocalDate settlementDate = fromApiJsonHelper.extractLocalDateNamed(ExternalTransferRequestParameters.SETTLEMENT_DATE, json);
         baseDataValidator.reset().parameter(ExternalTransferRequestParameters.SETTLEMENT_DATE).value(settlementDate).notNull();
 
+        final String transferExternalGroupId = fromApiJsonHelper
+                .extractStringNamed(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID, json);
+        baseDataValidator.reset().parameter(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID).value(transferExternalGroupId)
+                .ignoreIfNull().notExceedingLengthOf(100);
+
+        if (!dataValidationErrors.isEmpty()) {
+            throw new PlatformApiDataValidationException("validation.msg.validation.errors.exist", "Validation errors exist.",
+                    dataValidationErrors);
+        }
+    }
+
+    private void validateIntermediarySaleRequestBody(String apiRequestBodyAsJson) {
+        final Set<String> requestParameters = new HashSet<>(Arrays.asList(ExternalTransferRequestParameters.SETTLEMENT_DATE,
+                ExternalTransferRequestParameters.OWNER_EXTERNAL_ID, ExternalTransferRequestParameters.TRANSFER_EXTERNAL_ID,
+                ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID, ExternalTransferRequestParameters.PURCHASE_PRICE_RATIO,
+                ExternalTransferRequestParameters.DATEFORMAT, ExternalTransferRequestParameters.LOCALE));
+        final Type typeOfMap = new TypeToken<Map<String, Object>>() {
+
+        }.getType();
+        fromApiJsonHelper.checkForUnsupportedParameters(typeOfMap, apiRequestBodyAsJson, requestParameters);
+
+        final List<ApiParameterError> dataValidationErrors = new ArrayList<>();
+        final DataValidatorBuilder baseDataValidator = new DataValidatorBuilder(dataValidationErrors).resource("loantransfer");
+        final JsonElement json = fromApiJsonHelper.parse(apiRequestBodyAsJson);
+
+        String ownerExternalId = fromApiJsonHelper.extractStringNamed(ExternalTransferRequestParameters.OWNER_EXTERNAL_ID, json);
+        baseDataValidator.reset().parameter(ExternalTransferRequestParameters.OWNER_EXTERNAL_ID).value(ownerExternalId).notBlank()
+                .notExceedingLengthOf(100);
+
+        String transferExternalId = fromApiJsonHelper.extractStringNamed(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_ID, json);
+        baseDataValidator.reset().parameter(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_ID).value(transferExternalId).ignoreIfNull()
+                .notExceedingLengthOf(100);
+
+        String purchasePriceRatio = fromApiJsonHelper.extractStringNamed(ExternalTransferRequestParameters.PURCHASE_PRICE_RATIO, json);
+        baseDataValidator.reset().parameter(ExternalTransferRequestParameters.PURCHASE_PRICE_RATIO).value(purchasePriceRatio).notBlank()
+                .notExceedingLengthOf(50);
+
+        LocalDate settlementDate = fromApiJsonHelper.extractLocalDateNamed(ExternalTransferRequestParameters.SETTLEMENT_DATE, json);
+        baseDataValidator.reset().parameter(ExternalTransferRequestParameters.SETTLEMENT_DATE).value(settlementDate).notNull();
+
+        String transferExternalGroupId = fromApiJsonHelper.extractStringNamed(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID,
+                json);
+        baseDataValidator.reset().parameter(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID).value(transferExternalGroupId)
+                .ignoreIfNull().notExceedingLengthOf(100);
+
         if (!dataValidationErrors.isEmpty()) {
             throw new PlatformApiDataValidationException("validation.msg.validation.errors.exist", "Validation errors exist.",
                     dataValidationErrors);
@@ -352,6 +555,12 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         return StringUtils.isEmpty(transferExternalId) ? ExternalId.generate() : ExternalIdFactory.produce(transferExternalId);
     }
 
+    private ExternalId getTransferExternalGroupIdFromJson(JsonElement json) {
+        String transferExternalGroupId = fromApiJsonHelper.extractStringNamed(ExternalTransferRequestParameters.TRANSFER_EXTERNAL_GROUP_ID,
+                json);
+        return StringUtils.isEmpty(transferExternalGroupId) ? null : ExternalIdFactory.produce(transferExternalGroupId);
+    }
+
     private String getPurchasePriceRatioFromJson(JsonElement json) {
         return fromApiJsonHelper.extractStringNamed(ExternalTransferRequestParameters.PURCHASE_PRICE_RATIO, json);
     }
@@ -368,10 +577,4 @@ public class ExternalAssetOwnersWriteServiceImpl implements ExternalAssetOwnersW
         externalAssetOwner.setExternalId(ExternalIdFactory.produce(externalId));
         return externalAssetOwnerRepository.saveAndFlush(externalAssetOwner);
     }
-
-    private LoanIdAndExternalIdAndStatus fetchLoanDetails(Long loanId) {
-        Optional<LoanIdAndExternalIdAndStatus> loanIdAndExternalIdAndStatusResult = loanRepository
-                .findLoanIdAndExternalIdAndStatusByLoanId(loanId);
-        return loanIdAndExternalIdAndStatusResult.orElseThrow(() -> new LoanNotFoundException(loanId));
-    }
 }
