@@ -23,18 +23,39 @@ import java.math.BigDecimal;
 import java.time.LocalDate;
 import java.util.List;
 import java.util.Set;
-import org.apache.fineract.accounting.producttoaccountmapping.data.ChargeToGLAccountMapper;
+import org.apache.fineract.accounting.glaccount.data.GLAccountData;
 import org.apache.fineract.infrastructure.core.data.EnumOptionData;
 import org.apache.fineract.infrastructure.core.data.StringEnumOptionData;
-import org.apache.fineract.portfolio.delinquency.api.DelinquencyApiResourceSwagger.GetDelinquencyBucketsResponse;
+import org.apache.fineract.portfolio.delinquency.data.DelinquencyBucketData;
 
 /**
  * Created by Chirag Gupta on 12/27/17.
  */
-final class LoanProductsApiResourceSwagger {
+public final class LoanProductsApiResourceSwagger {
 
     private LoanProductsApiResourceSwagger() {}
 
+    @Schema(description = "LoanProductChargeData")
+    public static final class LoanProductChargeData {
+
+        private LoanProductChargeData() {}
+
+        @Schema(example = "1")
+        public Long id;
+        @Schema(example = "60.0")
+        public BigDecimal amount;
+    }
+
+    @Schema(description = "LoanProductChargeToGLAccountMapper")
+    public static final class LoanProductChargeToGLAccountMapper {
+
+        private LoanProductChargeToGLAccountMapper() {}
+
+        public LoanProductChargeData charge;
+        public GLAccountData incomeAccount;
+
+    }
+
     @Schema(description = "PostLoanProductsRequest")
     public static final class PostLoanProductsRequest {
 
@@ -134,6 +155,8 @@ final class LoanProductsApiResourceSwagger {
         public Integer daysInMonthType;
         @Schema(example = "1")
         public Integer daysInYearType;
+        @Schema(example = "FULL_LEAP_YEAR")
+        public String daysInYearCustomStrategy;
         @Schema(example = "true")
         public Boolean allowPartialPeriodInterestCalcualtion;
         @Schema(example = "179")
@@ -174,6 +197,22 @@ final class LoanProductsApiResourceSwagger {
         public Boolean enableAccrualActivityPosting;
         @Schema(example = "false")
         public Boolean interestRecognitionOnDisbursementDate;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT", allowableValues = "FLAT")
+        public String capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION", allowableValues = "EQUAL_AMORTIZATION")
+        public String capitalizedIncomeStrategy;
+        @Schema(example = "FEE", allowableValues = { "FEE", "INTEREST" })
+        public String capitalizedIncomeType;
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT", allowableValues = "FLAT")
+        public String buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION", allowableValues = "EQUAL_AMORTIZATION")
+        public String buyDownFeeStrategy;
+        @Schema(example = "FEE", allowableValues = { "FEE", "INTEREST" })
+        public String buyDownFeeIncomeType;
 
         // Interest Recalculation
         @Schema(example = "false")
@@ -248,10 +287,18 @@ final class LoanProductsApiResourceSwagger {
         public Long incomeFromGoodwillCreditFeesAccountId;
         @Schema(example = "11")
         public Long incomeFromGoodwillCreditPenaltyAccountId;
+        @Schema(example = "25")
+        public Long deferredIncomeLiabilityAccountId;
+        @Schema(example = "37")
+        public Long incomeFromCapitalizationAccountId;
+        @Schema(example = "27")
+        public Long buyDownExpenseAccountId;
+        @Schema(example = "38")
+        public Long incomeFromBuyDownAccountId;
         public List<GetLoanProductsProductIdResponse.GetLoanPaymentChannelToFundSourceMappings> paymentChannelToFundSourceMappings;
-        public List<GetLoanProductsProductIdResponse.GetLoanFeeToIncomeAccountMappings> feeToIncomeAccountMappings;
+        public List<LoanProductChargeToGLAccountMapper> feeToIncomeAccountMappings;
         public List<PostChargeOffReasonToExpenseAccountMappings> chargeOffReasonToExpenseAccountMappings;
-        public List<ChargeToGLAccountMapper> penaltyToIncomeAccountMappings;
+        public List<LoanProductChargeToGLAccountMapper> penaltyToIncomeAccountMappings;
 
         // Multi Disburse
         @Schema(example = "true")
@@ -265,7 +312,7 @@ final class LoanProductsApiResourceSwagger {
         @Schema(example = "36000.00")
         public Double outstandingLoanBalance;
 
-        public List<ChargeData> charges;
+        public List<LoanProductChargeData> charges;
         @Schema(example = "en_GB")
         public String locale;
         @Schema(example = "dd MMMM yyyy")
@@ -300,14 +347,6 @@ final class LoanProductsApiResourceSwagger {
             public boolean graceOnArrearsAgeing;
         }
 
-        static final class ChargeData {
-
-            private ChargeData() {}
-
-            @Schema(example = "1")
-            public Long id;
-        }
-
         static final class RateData {
 
             private RateData() {}
@@ -348,6 +387,30 @@ final class LoanProductsApiResourceSwagger {
         public Long resourceId;
     }
 
+    public static final class GetLoanProductsDaysInMonthType {
+
+        private GetLoanProductsDaysInMonthType() {}
+
+        @Schema(example = "1")
+        public Long id;
+        @Schema(example = "DaysInMonthType.actual")
+        public String code;
+        @Schema(example = "Actual")
+        public String description;
+    }
+
+    public static final class GetLoanProductsDaysInYearType {
+
+        private GetLoanProductsDaysInYearType() {}
+
+        @Schema(example = "1")
+        public Long id;
+        @Schema(example = "DaysInYearType.actual")
+        public String code;
+        @Schema(example = "Actual")
+        public String description;
+    }
+
     @Schema(description = "GetLoanProductsResponse")
     public static final class GetLoanProductsResponse {
 
@@ -433,27 +496,15 @@ final class LoanProductsApiResourceSwagger {
             public String description;
         }
 
-        static final class GetLoansProductsDaysInMonthType {
+        static final class GetLoanProductsDaysInYearCustomStrategy {
 
-            private GetLoansProductsDaysInMonthType() {}
+            private GetLoanProductsDaysInYearCustomStrategy() {}
 
-            @Schema(example = "30")
-            public Long id;
-            @Schema(example = "DaysInMonthType.days360")
+            @Schema(example = "FULL_LEAP_YEAR")
+            public String id;
+            @Schema(example = "DaysInYearCustomStrategyType.fullLeapYear")
             public String code;
-            @Schema(example = "30 Days")
-            public String description;
-        }
-
-        static final class GetLoansProductsDaysInYearType {
-
-            private GetLoansProductsDaysInYearType() {}
-
-            @Schema(example = "360")
-            public Long id;
-            @Schema(example = "DaysInYearType.days360")
-            public String code;
-            @Schema(example = "360 Days")
+            @Schema(example = "Full Leap Year")
             public String description;
         }
 
@@ -612,8 +663,9 @@ final class LoanProductsApiResourceSwagger {
         public List<Integer> interestRateVariationsForBorrowerCycle;
         @Schema(example = "[]")
         public List<Integer> numberOfRepaymentVariationsForBorrowerCycle;
-        public GetLoansProductsDaysInMonthType daysInMonthType;
-        public GetLoansProductsDaysInYearType daysInYearType;
+        public GetLoanProductsDaysInMonthType daysInMonthType;
+        public GetLoanProductsDaysInYearType daysInYearType;
+        public GetLoanProductsDaysInYearCustomStrategy daysInYearCustomStrategy;
         @Schema(example = "true")
         public Boolean isInterestRecalculationEnabled;
         public GetLoanProductsInterestRecalculationData interestRecalculationData;
@@ -623,6 +675,24 @@ final class LoanProductsApiResourceSwagger {
         public GetLoanProductsResponse.GetLoanProductsRepaymentStartDateType repaymentStartDateType;
         public List<StringEnumOptionData> supportedInterestRefundTypes;
         public StringEnumOptionData chargeOffBehaviour;
+        @Schema(example = "false")
+        public Boolean interestRecognitionOnDisbursementDate;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData capitalizedIncomeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData capitalizedIncomeType;
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData buyDownFeeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData buyDownFeeIncomeType;
     }
 
     @Schema(description = "GetLoanProductsTemplateResponse")
@@ -698,30 +768,6 @@ final class LoanProductsApiResourceSwagger {
             public String description;
         }
 
-        static final class GetLoansProductsDaysInMonthTemplateType {
-
-            private GetLoansProductsDaysInMonthTemplateType() {}
-
-            @Schema(example = "1")
-            public Long id;
-            @Schema(example = "DaysInMonthType.actual")
-            public String code;
-            @Schema(example = "Actual")
-            public String description;
-        }
-
-        static final class GetLoanProductsDaysInYearTemplateType {
-
-            private GetLoanProductsDaysInYearTemplateType() {}
-
-            @Schema(example = "1")
-            public Long id;
-            @Schema(example = "DaysInYearType.actual")
-            public String code;
-            @Schema(example = "Actual")
-            public String description;
-        }
-
         static final class GetLoanProductsInterestRecalculationTemplateData {
 
             private GetLoanProductsInterestRecalculationTemplateData() {}
@@ -1036,24 +1082,6 @@ final class LoanProductsApiResourceSwagger {
             public String description;
         }
 
-        static final class GetLoanProductsChargeOffReasonOptions {
-
-            private GetLoanProductsChargeOffReasonOptions() {}
-
-            @Schema(example = "2")
-            public Long id;
-            @Schema(example = "debit_card")
-            public String name;
-            @Schema(example = "2")
-            public Integer position;
-            @Schema(example = "Charge-Off reason description")
-            public String description;
-            @Schema(example = "true")
-            public Boolean active;
-            @Schema(example = "false")
-            public Boolean mandatory;
-        }
-
         @Schema(example = "false")
         public Boolean includeInBorrowerCycle;
         @Schema(example = "false")
@@ -1071,8 +1099,9 @@ final class LoanProductsApiResourceSwagger {
         @Schema(example = "[]")
         public List<Integer> numberOfRepaymentVariationsForBorrowerCycle;
         public GetLoanProductsTemplateResponse.GetLoanProductsAccountingRule accountingRule;
-        public GetLoansProductsDaysInMonthTemplateType daysInMonthType;
-        public GetLoanProductsDaysInYearTemplateType daysInYearType;
+        public GetLoanProductsDaysInMonthType daysInMonthType;
+        public GetLoanProductsDaysInYearType daysInYearType;
+        public StringEnumOptionData daysInYearCustomStrategy;
         @Schema(example = "false")
         public Boolean isInterestRecalculationEnabled;
         public GetLoanProductsInterestRecalculationTemplateData interestRecalculationData;
@@ -1089,8 +1118,9 @@ final class LoanProductsApiResourceSwagger {
         public Set<GetLoanProductsResponse.GetLoanProductsAccountingRule> accountingRuleOptions;
         public GetLoanProductsAccountingMappingOptions accountingMappingOptions;
         public Set<GetLoanProductsValueConditionTypeOptions> valueConditionTypeOptions;
-        public Set<GetLoansProductsDaysInMonthTemplateType> daysInMonthTypeOptions;
+        public Set<StringEnumOptionData> daysInMonthTypeOptions;
         public Set<GetLoanProductsInterestTemplateType> daysInYearTypeOptions;
+        public Set<StringEnumOptionData> daysInYearTypeCustomStrategyOptions;
         public Set<GetLoanProductsResponse.GetLoanProductsInterestRecalculationData.GetLoanProductsInterestRecalculationCompoundingType> interestRecalculationCompoundingTypeOptions;
         public Set<GetLoanProductsResponse.GetLoanProductsInterestRecalculationData.GetLoanProductsRescheduleStrategyType> rescheduleStrategyTypeOptions;
         public Set<GetLoanProductsResponse.GetLoanProductsInterestRecalculationData.GetLoanProductsInterestRecalculationCompoundingFrequencyType> interestRecalculationFrequencyTypeOptions;
@@ -1109,6 +1139,29 @@ final class LoanProductsApiResourceSwagger {
         public List<GetLoanProductsChargeOffReasonOptions> chargeOffReasonOptions;
         public StringEnumOptionData chargeOffBehaviour;
         public List<StringEnumOptionData> chargeOffBehaviourOptions;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData capitalizedIncomeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData capitalizedIncomeType;
+        public List<StringEnumOptionData> capitalizedIncomeCalculationTypeOptions;
+        public List<StringEnumOptionData> capitalizedIncomeStrategyOptions;
+        public List<StringEnumOptionData> capitalizedIncomeTypeOptions;
+
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData buyDownFeeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData buyDownFeeIncomeType;
+        public List<StringEnumOptionData> buyDownFeeCalculationTypeOptions;
+        public List<StringEnumOptionData> buyDownFeeStrategyOptions;
+        public List<StringEnumOptionData> buyDownFeeIncomeTypeOptions;
     }
 
     @Schema(description = "GetLoanProductsProductIdResponse")
@@ -1219,6 +1272,10 @@ final class LoanProductsApiResourceSwagger {
             public GetGlAccountMapping chargeOffExpenseAccount;
             public GetGlAccountMapping chargeOffFraudExpenseAccount;
             public GetGlAccountMapping overpaymentLiabilityAccount;
+            public GetGlAccountMapping deferredIncomeLiabilityAccount;
+            public GetGlAccountMapping incomeFromCapitalizationAccount;
+            public GetGlAccountMapping buyDownExpenseAccount;
+            public GetGlAccountMapping incomeFromBuyDownAccount;
         }
 
         static final class GetLoanPaymentChannelToFundSourceMappings {
@@ -1366,13 +1423,16 @@ final class LoanProductsApiResourceSwagger {
         @Schema(example = "false")
         public Boolean canDefineInstallmentAmount;
         @Schema(example = "[]")
-        public List<Integer> charges;
+        public List<LoanProductChargeData> charges;
         public Set<GetLoanProductsPrincipalVariationsForBorrowerCycle> productsPrincipalVariationsForBorrowerCycle;
         @Schema(example = "[]")
         public List<Integer> interestRateVariationsForBorrowerCycle;
         @Schema(example = "[]")
         public List<Integer> numberOfRepaymentVariationsForBorrowerCycle;
         public GetLoanProductsResponse.GetLoanProductsAccountingRule accountingRule;
+        public GetLoanProductsDaysInMonthType daysInMonthType;
+        public GetLoanProductsDaysInYearType daysInYearType;
+        public StringEnumOptionData daysInYearCustomStrategy;
         @Schema(example = "false")
         public Boolean canUseForTopup;
         public GetLoanAccountingMappings accountingMappings;
@@ -1397,8 +1457,8 @@ final class LoanProductsApiResourceSwagger {
         public Integer overdueDaysForNPA;
         @Schema(example = "50")
         public Integer principalThresholdForLastInstalment;
-        public GetDelinquencyBucketsResponse delinquencyBucket;
-        public List<GetDelinquencyBucketsResponse> delinquencyBucketOptions;
+        public DelinquencyBucketData delinquencyBucket;
+        public List<DelinquencyBucketData> delinquencyBucketOptions;
         @Schema(example = "false")
         public Boolean enableInstallmentLevelDelinquency;
         @Schema(example = "3")
@@ -1421,10 +1481,33 @@ final class LoanProductsApiResourceSwagger {
         @Schema(example = "false")
         public Boolean enableAccrualActivityPosting;
         public List<StringEnumOptionData> supportedInterestRefundTypes;
-        public List<GetLoanProductsTemplateResponse.GetLoanProductsChargeOffReasonOptions> chargeOffReasonOptions;
+        public List<GetLoanProductsChargeOffReasonOptions> chargeOffReasonOptions;
         public StringEnumOptionData chargeOffBehaviour;
         @Schema(example = "false")
         public Boolean interestRecognitionOnDisbursementDate;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData capitalizedIncomeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData capitalizedIncomeType;
+        public List<StringEnumOptionData> capitalizedIncomeCalculationTypeOptions;
+        public List<StringEnumOptionData> capitalizedIncomeStrategyOptions;
+        public List<StringEnumOptionData> capitalizedIncomeTypeOptions;
+
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData buyDownFeeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData buyDownFeeIncomeType;
+        public List<StringEnumOptionData> buyDownFeeCalculationTypeOptions;
+        public List<StringEnumOptionData> buyDownFeeStrategyOptions;
+        public List<StringEnumOptionData> buyDownFeeIncomeTypeOptions;
     }
 
     @Schema(description = "PutLoanProductsProductIdRequest")
@@ -1526,6 +1609,8 @@ final class LoanProductsApiResourceSwagger {
         public Long daysInMonthType;
         @Schema(example = "1")
         public Long daysInYearType;
+        @Schema(example = "FULL_LEAP_YEAR")
+        public String daysInYearCustomStrategy;
         @Schema(example = "true")
         public Boolean allowPartialPeriodInterestCalcualtion;
         @Schema(example = "179")
@@ -1590,6 +1675,8 @@ final class LoanProductsApiResourceSwagger {
         public Boolean isCompoundingToBePostedAsTransaction;
         @Schema(example = "false")
         public Boolean allowCompoundingOnEod;
+        @Schema(example = "false")
+        public Boolean disallowInterestCalculationOnPastDue;
 
         // Accounting
         @Schema(example = "3")
@@ -1640,10 +1727,18 @@ final class LoanProductsApiResourceSwagger {
 
         @Schema(example = "11")
         public Long incomeFromChargeOffPenaltyAccountId;
+        @Schema(example = "25")
+        public Long deferredIncomeLiabilityAccountId;
+        @Schema(example = "37")
+        public Long incomeFromCapitalizationAccountId;
+        @Schema(example = "27")
+        public Long buyDownExpenseAccountId;
+        @Schema(example = "38")
+        public Long incomeFromBuyDownAccountId;
         public List<GetLoanProductsProductIdResponse.GetLoanPaymentChannelToFundSourceMappings> paymentChannelToFundSourceMappings;
-        public List<GetLoanProductsProductIdResponse.GetLoanFeeToIncomeAccountMappings> feeToIncomeAccountMappings;
+        public List<LoanProductChargeToGLAccountMapper> feeToIncomeAccountMappings;
         public List<PostLoanProductsRequest.PostChargeOffReasonToExpenseAccountMappings> chargeOffReasonToExpenseAccountMappings;
-        public List<ChargeToGLAccountMapper> penaltyToIncomeAccountMappings;
+        public List<LoanProductChargeToGLAccountMapper> penaltyToIncomeAccountMappings;
         @Schema(example = "false")
         public Boolean enableAccrualActivityPosting;
 
@@ -1659,7 +1754,7 @@ final class LoanProductsApiResourceSwagger {
         @Schema(example = "36000.00")
         public Double outstandingLoanBalance;
 
-        public List<PostLoanProductsRequest.ChargeData> charges;
+        public List<LoanProductChargeData> charges;
         @Schema(example = "en_GB")
         public String locale;
         @Schema(example = "dd MMMM yyyy")
@@ -1673,55 +1768,25 @@ final class LoanProductsApiResourceSwagger {
         public PostLoanProductsRequest.AllowAttributeOverrides allowAttributeOverrides;
         public List<PostLoanProductsRequest.RateData> rates;
 
-        static final class AllowAttributeOverrides {
-
-            private AllowAttributeOverrides() {}
-
-            @Schema(example = "true")
-            public boolean amortizationType;
-            @Schema(example = "true")
-            public boolean interestType;
-            @Schema(example = "true")
-            public boolean transactionProcessingStrategyCode;
-            @Schema(example = "true")
-            public boolean interestCalculationPeriodType;
-            @Schema(example = "true")
-            public boolean inArrearsTolerance;
-            @Schema(example = "true")
-            public boolean repaymentEvery;
-            @Schema(example = "true")
-            public boolean graceOnPrincipalAndInterestPayment;
-            @Schema(example = "true")
-            public boolean graceOnArrearsAgeing;
-        }
-
-        static final class ChargeData {
-
-            private ChargeData() {}
-
-            @Schema(example = "1")
-            public Long id;
-        }
-
-        static final class RateData {
-
-            private RateData() {}
-
-            @Schema(example = "1")
-            public Long id;
-            @Schema(example = "some name")
-            public String name;
-            @Schema(example = "20")
-            public BigDecimal percentage;
-            @Schema(description = "Apply specific product using its id, code, and value.")
-            public EnumOptionData productApply;
-            @Schema(example = "false")
-            public boolean active;
-        }
-
         public List<String> supportedInterestRefundTypes;
         @Schema(example = "REGULAR")
         public String chargeOffBehaviour;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT", allowableValues = "FLAT")
+        public String capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION", allowableValues = "EQUAL_AMORTIZATION")
+        public String capitalizedIncomeStrategy;
+        @Schema(example = "FEE", allowableValues = { "FEE", "INTEREST" })
+        public String capitalizedIncomeType;
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT", allowableValues = "FLAT")
+        public String buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION", allowableValues = "EQUAL_AMORTIZATION")
+        public String buyDownFeeStrategy;
+        @Schema(example = "FEE", allowableValues = { "FEE", "INTEREST" })
+        public String buyDownFeeIncomeType;
     }
 
     public static final class AdvancedPaymentData {
@@ -1780,4 +1845,23 @@ final class LoanProductsApiResourceSwagger {
         public Long resourceId;
         public PutLoanChanges changes;
     }
+
+    @Schema(description = "GetLoanProductsChargeOffReasonOptions")
+    public static final class GetLoanProductsChargeOffReasonOptions {
+
+        private GetLoanProductsChargeOffReasonOptions() {}
+
+        @Schema(example = "2")
+        public Long id;
+        @Schema(example = "debit_card")
+        public String name;
+        @Schema(example = "2")
+        public Integer position;
+        @Schema(example = "Charge-Off reason description")
+        public String description;
+        @Schema(example = "true")
+        public Boolean active;
+        @Schema(example = "false")
+        public Boolean mandatory;
+    }
 }
