@@ -22,10 +22,11 @@ import io.swagger.v3.oas.annotations.media.Schema;
 import java.math.BigDecimal;
 import java.time.LocalDate;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 import org.apache.fineract.infrastructure.core.data.EnumOptionData;
 import org.apache.fineract.infrastructure.core.data.StringEnumOptionData;
-import org.apache.fineract.portfolio.delinquency.api.DelinquencyApiResourceSwagger.GetDelinquencyRangesResponse;
+import org.apache.fineract.portfolio.delinquency.data.DelinquencyRangeData;
 
 /**
  * Created by Chirag Gupta on 12/09/17.
@@ -42,9 +43,9 @@ final class LoansApiResourceSwagger {
         @Schema(example = "[2012, 4, 3]")
         public LocalDate approvalDate;
         @Schema(example = "200.000000")
-        public Double approvalAmount;
+        public BigDecimal approvalAmount;
         @Schema(example = "200.000000")
-        public Double netDisbursalAmount;
+        public BigDecimal netDisbursalAmount;
 
         public GetLoanCurrency currency;
     }
@@ -289,29 +290,29 @@ final class LoansApiResourceSwagger {
             @Schema(example = "30")
             public Long loanTermInDays;
             @Schema(example = "200.000000")
-            public Double totalPrincipalDisbursed;
+            public BigDecimal totalPrincipalDisbursed;
             @Schema(example = "200.00")
-            public Double totalPrincipalExpected;
+            public BigDecimal totalPrincipalExpected;
             @Schema(example = "200.00")
-            public Double totalPrincipalPaid;
+            public BigDecimal totalPrincipalPaid;
             @Schema(example = "0.00")
-            public Double totalInterestCharged;
+            public BigDecimal totalInterestCharged;
             @Schema(example = "0.00")
-            public Double totalFeeChargesCharged;
+            public BigDecimal totalFeeChargesCharged;
             @Schema(example = "0.00")
-            public Double totalPenaltyChargesCharged;
+            public BigDecimal totalPenaltyChargesCharged;
             @Schema(example = "0.00")
-            public Double totalWaived;
+            public BigDecimal totalWaived;
             @Schema(example = "0.00")
-            public Double totalWrittenOff;
+            public BigDecimal totalWrittenOff;
             @Schema(example = "200.00")
-            public Double totalRepaymentExpected;
+            public BigDecimal totalRepaymentExpected;
             @Schema(example = "200.00")
-            public Double totalPaidInAdvance;
+            public BigDecimal totalPaidInAdvance;
             @Schema(example = "0.00")
-            public Double totalPaidLate;
+            public BigDecimal totalPaidLate;
             @Schema(example = "0.00")
-            public Double totalOutstanding;
+            public BigDecimal totalOutstanding;
             public List<GetLoansLoanIdRepaymentPeriod> periods;
         }
 
@@ -332,71 +333,71 @@ final class LoansApiResourceSwagger {
             @Schema(example = "30")
             public Long daysInPeriod;
             @Schema(example = "200.000000")
-            public Double principalOriginalDue;
+            public BigDecimal principalOriginalDue;
             @Schema(example = "200.000000")
-            public Double principalDue;
+            public BigDecimal principalDue;
             @Schema(example = "200.000000")
-            public Double principalPaid;
+            public BigDecimal principalPaid;
             @Schema(example = "0.000000")
-            public Double principalWrittenOff;
+            public BigDecimal principalWrittenOff;
             @Schema(example = "20.000000")
-            public Double principalOutstanding;
+            public BigDecimal principalOutstanding;
             @Schema(example = "20.000000")
-            public Double principalLoanBalanceOutstanding;
+            public BigDecimal principalLoanBalanceOutstanding;
             @Schema(example = "0.000000")
-            public Double interestOriginalDue;
+            public BigDecimal interestOriginalDue;
             @Schema(example = "0.000000")
-            public Double interestDue;
+            public BigDecimal interestDue;
             @Schema(example = "0.000000")
-            public Double interestPaid;
+            public BigDecimal interestPaid;
             @Schema(example = "0.000000")
-            public Double interestWaived;
+            public BigDecimal interestWaived;
             @Schema(example = "0.000000")
-            public Double interestWrittenOff;
+            public BigDecimal interestWrittenOff;
             @Schema(example = "0.000000")
-            public Double interestOutstanding;
+            public BigDecimal interestOutstanding;
             @Schema(example = "0.000000")
-            public Double feeChargesDue;
+            public BigDecimal feeChargesDue;
             @Schema(example = "20.000000")
-            public Double feeChargesPaid;
+            public BigDecimal feeChargesPaid;
             @Schema(example = "20.000000")
-            public Double feeChargesWaived;
+            public BigDecimal feeChargesWaived;
             @Schema(example = "20.000000")
-            public Double feeChargesWrittenOff;
+            public BigDecimal feeChargesWrittenOff;
             @Schema(example = "20.000000")
-            public Double feeChargesOutstanding;
+            public BigDecimal feeChargesOutstanding;
             @Schema(example = "20.000000")
-            public Double penaltyChargesDue;
+            public BigDecimal penaltyChargesDue;
             @Schema(example = "20.000000")
-            public Double penaltyChargesPaid;
+            public BigDecimal penaltyChargesPaid;
             @Schema(example = "20.000000")
-            public Double penaltyChargesWaived;
+            public BigDecimal penaltyChargesWaived;
             @Schema(example = "20.000000")
-            public Double penaltyChargesWrittenOff;
+            public BigDecimal penaltyChargesWrittenOff;
             @Schema(example = "20.000000")
-            public Double penaltyChargesOutstanding;
+            public BigDecimal penaltyChargesOutstanding;
             @Schema(example = "20.000000")
-            public Double totalOriginalDueForPeriod;
+            public BigDecimal totalOriginalDueForPeriod;
             @Schema(example = "20.000000")
-            public Double totalDueForPeriod;
+            public BigDecimal totalDueForPeriod;
             @Schema(example = "20.000000")
-            public Double totalPaidForPeriod;
+            public BigDecimal totalPaidForPeriod;
             @Schema(example = "20.000000")
-            public Double totalPaidInAdvanceForPeriod;
+            public BigDecimal totalPaidInAdvanceForPeriod;
             @Schema(example = "20.000000")
-            public Double totalPaidLateForPeriod;
+            public BigDecimal totalPaidLateForPeriod;
             @Schema(example = "20.000000")
-            public Double totalWaivedForPeriod;
+            public BigDecimal totalWaivedForPeriod;
             @Schema(example = "20.000000")
-            public Double totalWrittenOffForPeriod;
+            public BigDecimal totalWrittenOffForPeriod;
             @Schema(example = "200.000000")
-            public Double totalOutstandingForPeriod;
+            public BigDecimal totalOutstandingForPeriod;
             @Schema(example = "20.000000")
-            public Double totalActualCostOfLoanForPeriod;
+            public BigDecimal totalActualCostOfLoanForPeriod;
             @Schema(example = "200.000000")
-            public Double totalInstallmentAmountForPeriod;
+            public BigDecimal totalInstallmentAmountForPeriod;
             @Schema(example = "2.000000")
-            public Double totalCredits;
+            public BigDecimal totalCredits;
             @Schema(example = "true")
             public Boolean downPaymentPeriod;
         }
@@ -412,15 +413,15 @@ final class LoansApiResourceSwagger {
             @Schema(example = "[2022, 07, 01]")
             public LocalDate actualDisbursementDate;
             @Schema(example = "22000.000000")
-            public Double principal;
+            public BigDecimal principal;
             @Schema(example = "22000.000000")
-            public Double netDisbursalAmount;
+            public BigDecimal netDisbursalAmount;
             @Schema(example = "1")
             public String loanChargeId;
             @Schema(example = "22000.000000")
-            public Double chargeAmount;
+            public BigDecimal chargeAmount;
             @Schema(example = "22000.000000")
-            public Double waivedChargeAmount;
+            public BigDecimal waivedChargeAmount;
             @Schema(example = "dd MMMM yyyy")
             public String dateFormat;
             @Schema(example = "de_DE")
@@ -517,77 +518,81 @@ final class LoansApiResourceSwagger {
 
             public GetLoansLoanIdCurrency currency;
             @Schema(example = "1000000.000000")
-            public Double principalDisbursed;
-            @Schema(example = "0.000000")
-            public Double principalPaid;
-            @Schema(example = "0.00")
-            public Double principalAdjustments;
-            @Schema(example = "0.000000")
-            public Double principalWrittenOff;
+            public BigDecimal totalPrincipal;
             @Schema(example = "1000000.000000")
-            public Double principalOutstanding;
+            public BigDecimal principalDisbursed;
+            @Schema(example = "1000000.000000")
+            public BigDecimal totalCapitalizedIncome;
+            @Schema(example = "0.000000")
+            public BigDecimal principalPaid;
+            @Schema(example = "0.00")
+            public BigDecimal principalAdjustments;
+            @Schema(example = "0.000000")
+            public BigDecimal principalWrittenOff;
+            @Schema(example = "1000000.000000")
+            public BigDecimal principalOutstanding;
             @Schema(example = "833333.300000")
-            public Double principalOverdue;
+            public BigDecimal principalOverdue;
             @Schema(example = "240000.000000")
-            public Double interestCharged;
+            public BigDecimal interestCharged;
             @Schema(example = "0.000000")
-            public Double interestPaid;
+            public BigDecimal interestPaid;
             @Schema(example = "0.000000")
-            public Double interestWaived;
+            public BigDecimal interestWaived;
             @Schema(example = "0.000000")
-            public Double interestWrittenOff;
+            public BigDecimal interestWrittenOff;
             @Schema(example = "240000.000000")
-            public Double interestOutstanding;
+            public BigDecimal interestOutstanding;
             @Schema(example = "200000.000000")
-            public Double interestOverdue;
+            public BigDecimal interestOverdue;
             @Schema(example = "0.00")
-            public Double feeAdjustments;
+            public BigDecimal feeAdjustments;
             @Schema(example = "18000.000000")
-            public Double feeChargesCharged;
+            public BigDecimal feeChargesCharged;
             @Schema(example = "0.000000")
-            public Double feeChargesDueAtDisbursementCharged;
+            public BigDecimal feeChargesDueAtDisbursementCharged;
             @Schema(example = "0.000000")
-            public Double feeChargesPaid;
+            public BigDecimal feeChargesPaid;
             @Schema(example = "0.000000")
-            public Double feeChargesWaived;
+            public BigDecimal feeChargesWaived;
             @Schema(example = "0.000000")
-            public Double feeChargesWrittenOff;
+            public BigDecimal feeChargesWrittenOff;
             @Schema(example = "18000.000000")
-            public Double feeChargesOutstanding;
+            public BigDecimal feeChargesOutstanding;
             @Schema(example = "15000.000000")
-            public Double feeChargesOverdue;
+            public BigDecimal feeChargesOverdue;
             @Schema(example = "0.00")
-            public Double penaltyAdjustments;
+            public BigDecimal penaltyAdjustments;
             @Schema(example = "0.000000")
-            public Double penaltyChargesCharged;
+            public BigDecimal penaltyChargesCharged;
             @Schema(example = "0.000000")
-            public Double penaltyChargesPaid;
+            public BigDecimal penaltyChargesPaid;
             @Schema(example = "0.000000")
-            public Double penaltyChargesWaived;
+            public BigDecimal penaltyChargesWaived;
             @Schema(example = "0.000000")
-            public Double penaltyChargesWrittenOff;
+            public BigDecimal penaltyChargesWrittenOff;
             @Schema(example = "0.000000")
-            public Double penaltyChargesOutstanding;
+            public BigDecimal penaltyChargesOutstanding;
             @Schema(example = "0.000000")
-            public Double penaltyChargesOverdue;
+            public BigDecimal penaltyChargesOverdue;
             @Schema(example = "1258000.000000")
-            public Double totalExpectedRepayment;
+            public BigDecimal totalExpectedRepayment;
             @Schema(example = "0.000000")
-            public Double totalRepayment;
+            public BigDecimal totalRepayment;
             @Schema(example = "258000.000000")
-            public Double totalExpectedCostOfLoan;
+            public BigDecimal totalExpectedCostOfLoan;
             @Schema(example = "0.000000")
-            public Double totalCostOfLoan;
+            public BigDecimal totalCostOfLoan;
             @Schema(example = "0.000000")
-            public Double totalWaived;
+            public BigDecimal totalWaived;
             @Schema(example = "0.000000")
-            public Double totalWrittenOff;
+            public BigDecimal totalWrittenOff;
             @Schema(example = "1258000.000000")
-            public Double totalOutstanding;
+            public BigDecimal totalOutstanding;
             @Schema(example = "1048333.30000")
-            public Double totalOverdue;
+            public BigDecimal totalOverdue;
             @Schema(example = "2456.30000")
-            public Double totalRecovered;
+            public BigDecimal totalRecovered;
             @Schema(example = "[2012, 5, 10]")
             public LocalDate overdueSinceDate;
             @Schema(example = "1")
@@ -597,9 +602,9 @@ final class LoansApiResourceSwagger {
             public GetLoansLoanIdLinkedAccount linkedAccount;
             public Set<GetLoansLoanIdDisbursementDetails> disbursementDetails;
             @Schema(example = "1100.000000")
-            public Double fixedEmiAmount;
+            public BigDecimal fixedEmiAmount;
             @Schema(example = "35000.000000")
-            public Double maxOutstandingLoanBalance;
+            public BigDecimal maxOutstandingLoanBalance;
             @Schema(example = "false")
             public Boolean canDisburse;
             @Schema(example = "true")
@@ -607,33 +612,33 @@ final class LoansApiResourceSwagger {
             @Schema(example = "false")
             public Boolean isNPA;
             @Schema(example = "0.000000")
-            public Double totalMerchantRefund;
+            public BigDecimal totalMerchantRefund;
             @Schema(example = "0.000000")
-            public Double totalMerchantRefundReversed;
+            public BigDecimal totalMerchantRefundReversed;
             @Schema(example = "0.000000")
-            public Double totalPayoutRefund;
+            public BigDecimal totalPayoutRefund;
             @Schema(example = "0.000000")
-            public Double totalPayoutRefundReversed;
+            public BigDecimal totalPayoutRefundReversed;
             @Schema(example = "0.000000")
-            public Double totalGoodwillCredit;
+            public BigDecimal totalGoodwillCredit;
             @Schema(example = "0.000000")
-            public Double totalGoodwillCreditReversed;
+            public BigDecimal totalGoodwillCreditReversed;
             @Schema(example = "0.000000")
-            public Double totalChargeAdjustment;
+            public BigDecimal totalChargeAdjustment;
             @Schema(example = "0.000000")
-            public Double totalChargeAdjustmentReversed;
+            public BigDecimal totalChargeAdjustmentReversed;
             @Schema(example = "0.000000")
-            public Double totalChargeback;
+            public BigDecimal totalChargeback;
             @Schema(example = "0.000000")
-            public Double totalCreditBalanceRefund;
+            public BigDecimal totalCreditBalanceRefund;
             @Schema(example = "0.000000")
-            public Double totalCreditBalanceRefundReversed;
+            public BigDecimal totalCreditBalanceRefundReversed;
             @Schema(example = "0.000000")
-            public Double totalRepaymentTransaction;
+            public BigDecimal totalRepaymentTransaction;
             @Schema(example = "0.000000")
-            public Double totalInterestPaymentWaiver;
+            public BigDecimal totalInterestPaymentWaiver;
             @Schema(example = "0.000000")
-            public Double totalRepaymentTransactionReversed;
+            public BigDecimal totalRepaymentTransactionReversed;
             @Schema(example = "0.000000")
             public BigDecimal totalUnpaidPayableDueInterest;
             @Schema(example = "0.000000")
@@ -722,6 +727,22 @@ final class LoansApiResourceSwagger {
                 public boolean chargeAdjustment;
                 @Schema(example = "false")
                 public boolean chargeoff;
+                @Schema(example = "false")
+                public boolean capitalizedIncome;
+                @Schema(example = "false")
+                public boolean capitalizedIncomeAmortization;
+                @Schema(example = "false")
+                public boolean capitalizedIncomeAdjustment;
+                @Schema(example = "false")
+                public boolean contractTermination;
+                @Schema(example = "false")
+                public boolean buyDownFee;
+                @Schema(example = "false")
+                public boolean buyDownFeeAdjustment;
+                @Schema(example = "false")
+                public boolean buyDownFeeAmortization;
+                @Schema(example = "false")
+                public boolean buyDownFeeAmortizationAdjustment;
             }
 
             static final class GetLoansLoanIdPaymentDetailData {
@@ -750,7 +771,7 @@ final class LoansApiResourceSwagger {
                 @Schema(example = "11")
                 public Long id;
                 @Schema(example = "100.000000")
-                public Double amount;
+                public BigDecimal amount;
                 @Schema(example = "9679")
                 public Integer installmentNumber;
                 @Schema(example = "1")
@@ -806,7 +827,7 @@ final class LoansApiResourceSwagger {
                 @Schema(example = "CHARGEBACK")
                 public String relationType;
                 @Schema(example = "100.00")
-                public Double amount;
+                public BigDecimal amount;
                 @Schema(example = "Repayment Adjustment Chargeback")
                 public String paymentType;
 
@@ -827,27 +848,27 @@ final class LoansApiResourceSwagger {
             @Schema(description = "Payment detail")
             public GetLoansLoanIdPaymentDetailData paymentDetailData;
             @Schema(example = "100.000000")
-            public Double amount;
+            public BigDecimal amount;
             @Schema(example = "100.000000")
-            public Double netDisbursalAmount;
+            public BigDecimal netDisbursalAmount;
             @Schema(example = "100.000000")
-            public Double principalPortion;
+            public BigDecimal principalPortion;
             @Schema(example = "100.000000")
-            public Double interestPortion;
+            public BigDecimal interestPortion;
             @Schema(example = "100.000000")
-            public Double feeChargesPortion;
+            public BigDecimal feeChargesPortion;
             @Schema(example = "100.000000")
-            public Double penaltyChargesPortion;
+            public BigDecimal penaltyChargesPortion;
             @Schema(example = "100.000000")
-            public Double overpaymentPortion;
+            public BigDecimal overpaymentPortion;
             @Schema(example = "100.000000")
-            public Double unrecognizedIncomePortion;
+            public BigDecimal unrecognizedIncomePortion;
             @Schema(example = "3")
             public String externalId;
             @Schema(example = "100.000000")
-            public Double fixedEmiAmount;
+            public BigDecimal fixedEmiAmount;
             @Schema(example = "100.000000")
-            public Double outstandingLoanBalance;
+            public BigDecimal outstandingLoanBalance;
             @Schema(example = "[2022, 07, 01]")
             public LocalDate submittedOnDate;
             public boolean manuallyReversed;
@@ -866,7 +887,7 @@ final class LoansApiResourceSwagger {
             @Schema(example = "de_DE")
             public String locale;
             @Schema(example = "100.000000")
-            public Double transactionAmount;
+            public BigDecimal transactionAmount;
             @Schema(example = "[2022, 07, 01]")
             public LocalDate transactionDate;
             @Schema(example = "101")
@@ -920,19 +941,19 @@ final class LoansApiResourceSwagger {
                 @Schema(example = "[2022, 07, 01]")
                 public LocalDate dueDate;
                 @Schema(example = "13.560000")
-                public Double amount;
+                public BigDecimal amount;
                 @Schema(example = "13.560000")
-                public Double amountOutstanding;
+                public BigDecimal amountOutstanding;
                 @Schema(example = "13.560000")
-                public Double amountWaived;
+                public BigDecimal amountWaived;
                 @Schema(example = "false")
                 public boolean paid;
                 @Schema(example = "false")
                 public boolean waived;
                 @Schema(example = "13.560000")
-                public Double amountAccrued;
+                public BigDecimal amountAccrued;
                 @Schema(example = "13.560000")
-                public Double amountUnrecognized;
+                public BigDecimal amountUnrecognized;
             }
 
             @Schema(example = "3")
@@ -948,23 +969,23 @@ final class LoansApiResourceSwagger {
             @Schema(description = "Enum option data")
             public GetLoansLoanIdEnumOptionData chargeCalculationType;
             @Schema(example = "3.400000")
-            public Double percentage;
+            public BigDecimal percentage;
             @Schema(example = "13.560000")
-            public Double amountPercentageAppliedTo;
+            public BigDecimal amountPercentageAppliedTo;
             @Schema(description = "currency")
             public GetLoansLoanIdCurrency currency;
             @Schema(example = "102.000000")
-            public Double amount;
+            public BigDecimal amount;
             @Schema(example = "12.000000")
-            public Double amountPaid;
+            public BigDecimal amountPaid;
             @Schema(example = "14.000000")
-            public Double amountWaived;
+            public BigDecimal amountWaived;
             @Schema(example = "102.000000")
-            public Double amountWrittenOff;
+            public BigDecimal amountWrittenOff;
             @Schema(example = "102.000000")
-            public Double amountOutstanding;
+            public BigDecimal amountOutstanding;
             @Schema(example = "102.000000")
-            public Double amountOrPercentage;
+            public BigDecimal amountOrPercentage;
             @Schema(example = "false")
             public boolean penalty;
             @Schema(description = "Enum option data")
@@ -978,15 +999,15 @@ final class LoansApiResourceSwagger {
             @Schema(example = "3")
             public Long loanId;
             @Schema(example = "30.000000")
-            public Double minCap;
+            public BigDecimal minCap;
             @Schema(example = "30.000000")
-            public Double maxCap;
+            public BigDecimal maxCap;
             @Schema(description = "List of GetLoansLoanIdLoanInstallmentChargeData")
             public List<GetLoansLoanIdLoanInstallmentChargeData> installmentChargeData;
             @Schema(example = "30.000000")
-            private Double amountAccrued;
+            private BigDecimal amountAccrued;
             @Schema(example = "30.000000")
-            private Double amountUnrecognized;
+            private BigDecimal amountUnrecognized;
             @Schema(example = "3ert3453")
             private String externalId;
         }
@@ -996,7 +1017,7 @@ final class LoansApiResourceSwagger {
             private GetLoansLoanIdDelinquencySummary() {}
 
             @Schema(example = "100.000000")
-            public Double availableDisbursementAmount;
+            public BigDecimal availableDisbursementAmount;
             @Schema(example = "12")
             public Integer pastDueDays;
             @Schema(example = "[2022, 07, 01]")
@@ -1006,7 +1027,7 @@ final class LoansApiResourceSwagger {
             @Schema(example = "[2022, 07, 01]")
             public LocalDate delinquentDate;
             @Schema(example = "100.000000")
-            public Double delinquentAmount;
+            public BigDecimal delinquentAmount;
             @Schema(example = "80.000000")
             public BigDecimal delinquentPrincipal;
             @Schema(example = "10.000000")
@@ -1018,12 +1039,12 @@ final class LoansApiResourceSwagger {
             @Schema(example = "[2022, 07, 01]")
             public LocalDate lastPaymentDate;
             @Schema(example = "100.000000")
-            public Double lastPaymentAmount;
+            public BigDecimal lastPaymentAmount;
 
             @Schema(example = "[2022, 07, 01]")
             public LocalDate lastRepaymentDate;
             @Schema(example = "100.000000")
-            public Double lastRepaymentAmount;
+            public BigDecimal lastRepaymentAmount;
 
             @Schema(description = "List of GetLoansLoanIdDelinquencyPausePeriod")
             public List<GetLoansLoanIdDelinquencyPausePeriod> delinquencyPausePeriods;
@@ -1090,7 +1111,7 @@ final class LoansApiResourceSwagger {
             @Schema(example = "[2024, 1, 1]")
             public LocalDate termVariationApplicableFrom;
             @Schema(example = "200.000000")
-            public Double decimalValue;
+            public BigDecimal decimalValue;
             @Schema(example = "[2024, 1, 1]")
             public LocalDate dateValue;
             @Schema(example = "false")
@@ -1135,11 +1156,11 @@ final class LoansApiResourceSwagger {
         @Schema(example = "1000000.00")
         public BigDecimal principal;
         @Schema(example = "1000.000000")
-        public Double approvedPrincipal;
+        public BigDecimal approvedPrincipal;
         @Schema(example = "1001.000000")
-        public Double proposedPrincipal;
+        public BigDecimal proposedPrincipal;
         @Schema(example = "200.000000")
-        public Double netDisbursalAmount;
+        public BigDecimal netDisbursalAmount;
         @Schema(example = "12")
         public Integer termFrequency;
         public GetLoansLoanIdTermPeriodFrequencyType termPeriodFrequencyType;
@@ -1175,13 +1196,13 @@ final class LoansApiResourceSwagger {
         public GetLoansLoanIdDelinquencySummary delinquent;
         @Schema(description = "Set of charges")
         public List<GetLoansLoanIdLoanChargeData> charges;
-        public GetDelinquencyRangesResponse delinquencyRange;
+        public DelinquencyRangeData delinquencyRange;
         @Schema(example = "false")
         public Boolean fraud;
         @Schema(example = "false")
         public Boolean enableInstallmentLevelDelinquency;
         @Schema(example = "250.000000")
-        public Double totalOverpaid;
+        public BigDecimal totalOverpaid;
         public LocalDate lastClosedBusinessDate;
         @Schema(example = "[2013, 11, 1]")
         public LocalDate overpaidOnDate;
@@ -1206,6 +1227,22 @@ final class LoansApiResourceSwagger {
         public StringEnumOptionData chargeOffBehaviour;
         @Schema(example = "false")
         public Boolean interestRecognitionOnDisbursementDate;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData capitalizedIncomeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData capitalizedIncomeType;
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT")
+        public StringEnumOptionData buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION")
+        public StringEnumOptionData buyDownFeeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData buyDownFeeIncomeType;
     }
 
     @Schema(description = "GetLoansResponse")
@@ -1225,10 +1262,20 @@ final class LoansApiResourceSwagger {
 
             private PostLoansDisbursementData() {}
 
-            @Schema(example = "[2013, 11, 1]")
-            public LocalDate expectedDisbursementDate;
+            @Schema(example = "1 November 2023")
+            public String expectedDisbursementDate;
             @Schema(example = "1000.00")
-            public Double principal;
+            public BigDecimal principal;
+        }
+
+        static final class PostLoansDataTable {
+
+            private PostLoansDataTable() {}
+
+            @Schema(example = "m_loan")
+            public String registeredTableName;
+            @Schema(example = "Datatable data")
+            public Map<String, Object> data;
         }
 
         private PostLoansRequest() {}
@@ -1273,6 +1320,8 @@ final class LoansApiResourceSwagger {
         public String transactionProcessingStrategyCode;
         @Schema(example = "360", allowableValues = "1, 360, 364, 36")
         public Integer daysInYearType;
+        @Schema(example = "FULL_LEAP_YEAR", allowableValues = "FULL_LEAP_YEAR, FEB_29_PERIOD_ONLY")
+        public String daysInYearCustomStrategy;
         @Schema(example = "individual")
         public String loanType;
         @Schema(example = "20 September 2011")
@@ -1307,6 +1356,24 @@ final class LoansApiResourceSwagger {
         public BigDecimal fixedEmiAmount;
         @Schema(example = "false")
         public Boolean interestRecognitionOnDisbursementDate;
+        @Schema(example = "false")
+        public Boolean enableIncomeCapitalization;
+        @Schema(example = "FLAT", allowableValues = "FLAT")
+        public String capitalizedIncomeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION", allowableValues = "EQUAL_AMORTIZATION")
+        public String capitalizedIncomeStrategy;
+        @Schema(example = "FEE")
+        public StringEnumOptionData capitalizedIncomeType;
+        @Schema(example = "false")
+        public Boolean enableBuyDownFee;
+        @Schema(example = "FLAT", allowableValues = "FLAT")
+        public String buyDownFeeCalculationType;
+        @Schema(example = "EQUAL_AMORTIZATION", allowableValues = "EQUAL_AMORTIZATION")
+        public String buyDownFeeStrategy;
+        @Schema(example = "FEE", allowableValues = { "FEE", "INTEREST" })
+        public String buyDownFeeIncomeType;
+        @Schema(example = "List of PostLoansDataTable")
+        public List<PostLoansDataTable> datatables;
 
         public List<PostLoansRequestChargeData> charges;
 
@@ -1365,7 +1432,7 @@ final class LoansApiResourceSwagger {
         @Schema(example = "0")
         public Long totalPrincipalPaid;
         @Schema(example = "13471.52")
-        public Double totalInterestCharged;
+        public BigDecimal totalInterestCharged;
         @Schema(example = "0")
         public Long totalFeeChargesCharged;
         @Schema(example = "0")
@@ -1375,7 +1442,7 @@ final class LoansApiResourceSwagger {
         @Schema(example = "0")
         public Long totalWrittenOff;
         @Schema(example = "113471.52")
-        public Double totalRepaymentExpected;
+        public BigDecimal totalRepaymentExpected;
         @Schema(example = "0")
         public Long totalRepayment;
         @Schema(example = "0")
@@ -1594,11 +1661,11 @@ final class LoansApiResourceSwagger {
             @Schema(example = "[2012, 4, 3]")
             public LocalDate expectedDisbursementDate;
             @Schema(example = "22000")
-            public Double principal;
+            public BigDecimal principal;
         }
 
         @Schema(example = "2")
-        public Integer toLoanOfficerId;
+        public Long toLoanOfficerId;
         @Schema(example = "02 September 2014")
         public String assignmentDate;
 
@@ -1609,7 +1676,7 @@ final class LoansApiResourceSwagger {
         @Schema(example = "dd MMMM yyyy")
         public String dateFormat;
         @Schema(example = "")
-        public Integer fromLoanOfficerId;
+        public Long fromLoanOfficerId;
         @Schema(example = "3e7791ce-aa10-11ec-b909-0242ac120002")
         public String externalId;
         @Schema(example = "5000.33")
@@ -1688,7 +1755,7 @@ final class LoansApiResourceSwagger {
         }
 
         @Schema(example = "2")
-        public Integer officeId;
+        public Long officeId;
         @Schema(example = "6")
         public Long clientId;
         @Schema(example = "3")
