@@ -40,7 +40,7 @@ final class SavingsProductsApiResourceSwagger {
             private PostSavingsCharges() {}
 
             @Schema(example = "1")
-            public Integer id;
+            public Long id;
         }
 
         @Schema(example = "Passbook Savings")
@@ -69,9 +69,19 @@ final class SavingsProductsApiResourceSwagger {
         public Integer interestCalculationDaysInYearType;
         @Schema(example = "1")
         public Integer accountingRule;
-        public Set<PostSavingsCharges> charges;
+        public List<PostSavingsCharges> charges;
         @Schema(example = "accountMappingForPayment")
         public String accountMappingForPayment;
+        @Schema(example = "false")
+        public Boolean withdrawalFeeForTransfers;
+        @Schema(example = "false")
+        public Boolean enforceMinRequiredBalance;
+        @Schema(example = "false")
+        public Boolean allowOverdraft;
+        @Schema(example = "false")
+        public Boolean withHoldTax;
+        @Schema(example = "false")
+        public Boolean isDormancyTrackingActive;
     }
 
     @Schema(description = "PostSavingsProductsResponse")
@@ -80,7 +90,7 @@ final class SavingsProductsApiResourceSwagger {
         private PostSavingsProductsResponse() {}
 
         @Schema(example = "1")
-        public Integer resourceId;
+        public Long resourceId;
     }
 
     @Schema(description = "PutSavingsProductsProductIdRequest")
@@ -114,7 +124,7 @@ final class SavingsProductsApiResourceSwagger {
         }
 
         @Schema(example = "1")
-        public Integer resourceId;
+        public Long resourceId;
         public PutSavingsChanges changes;
     }
 
@@ -231,7 +241,7 @@ final class SavingsProductsApiResourceSwagger {
             private GetSavingsProductsGlAccount() {}
 
             @Schema(example = "12")
-            public Integer id;
+            public Long id;
             @Schema(example = "savings control")
             public String name;
             @Schema(example = "2000001")
@@ -265,7 +275,7 @@ final class SavingsProductsApiResourceSwagger {
                 private GetSavingsProductsPaymentType() {}
 
                 @Schema(example = "10")
-                public Integer id;
+                public Long id;
                 @Schema(example = "check")
                 public String name;
             }
@@ -275,7 +285,7 @@ final class SavingsProductsApiResourceSwagger {
                 private GetSavingsProductsFundSourceAccount() {}
 
                 @Schema(example = "12")
-                public Integer id;
+                public Long id;
                 @Schema(example = "savings ref")
                 public String name;
                 @Schema(example = "20")
@@ -295,7 +305,7 @@ final class SavingsProductsApiResourceSwagger {
                 private GetSavingsProductsFeeToIncomeAccountMappingsCharge() {}
 
                 @Schema(example = "11")
-                public Integer id;
+                public Long id;
                 @Schema(example = "sav charge")
                 public String name;
                 @Schema(example = "false")
@@ -309,7 +319,7 @@ final class SavingsProductsApiResourceSwagger {
                 private GetSavingsProductsFeeToIncomeAccountMappingsIncomeAccount() {}
 
                 @Schema(example = "16")
-                public Integer id;
+                public Long id;
                 @Schema(example = "income from savings fee")
                 public String name;
                 @Schema(example = "24")
@@ -329,7 +339,7 @@ final class SavingsProductsApiResourceSwagger {
                 private GetSavingsProductsPenaltyToIncomeAccountMappingsCharge() {}
 
                 @Schema(example = "12")
-                public Integer id;
+                public Long id;
                 @Schema(example = "sav 2")
                 public String name;
                 @Schema(example = "false")
@@ -343,7 +353,7 @@ final class SavingsProductsApiResourceSwagger {
         }
 
         @Schema(example = "1")
-        public Integer id;
+        public Long id;
         @Schema(example = "savings product")
         public String name;
         @Schema(example = "sa1")
@@ -365,7 +375,7 @@ final class SavingsProductsApiResourceSwagger {
         public Set<GetSavingsProductsFeeToIncomeAccountMappings> feeToIncomeAccountMappings;
         public Set<GetSavingsProductsPenaltyToIncomeAccountMappings> penaltyToIncomeAccountMappings;
         @Schema(example = "[]")
-        public List<Integer> charges;
+        public List<Long> charges;
     }
 
     @Schema(description = "GetSavingsProductsTemplateResponse")
@@ -462,11 +472,11 @@ final class SavingsProductsApiResourceSwagger {
                     private GetSavingsProductsLiabilityTagId() {}
 
                     @Schema(example = "0")
-                    public Integer id;
+                    public Long id;
                 }
 
                 @Schema(example = "15")
-                public Integer id;
+                public Long id;
                 @Schema(example = "Savings Control")
                 public String name;
                 @Schema(example = "50001")
@@ -504,7 +514,7 @@ final class SavingsProductsApiResourceSwagger {
                 }
 
                 @Schema(example = "2")
-                public Integer id;
+                public Long id;
                 @Schema(example = "Cash")
                 public String name;
                 @Schema(example = "100001")
@@ -535,7 +545,7 @@ final class SavingsProductsApiResourceSwagger {
                 }
 
                 @Schema(example = "6")
-                public Integer id;
+                public Long id;
                 @Schema(example = "Write Off Expenses")
                 public String name;
                 @Schema(example = "60001")
@@ -566,7 +576,7 @@ final class SavingsProductsApiResourceSwagger {
                 }
 
                 @Schema(example = "3")
-                public Integer id;
+                public Long id;
                 @Schema(example = "income from interest")
                 public String name;
                 @Schema(example = "40001")
@@ -649,7 +659,7 @@ final class SavingsProductsApiResourceSwagger {
             public GetSavingsChargeTimeType chargeTimeType;
             public GetSavingsProductsResponse.GetSavingsCurrency currency;
             @Schema(example = "4")
-            public Integer id;
+            public Long id;
             @Schema(example = "Savings charge 1")
             public String name;
             @Schema(example = "false")
@@ -683,6 +693,6 @@ final class SavingsProductsApiResourceSwagger {
         private DeleteSavingsProductsProductIdResponse() {}
 
         @Schema(example = "1")
-        public Integer resourceId;
+        public Long resourceId;
     }
 }
