@@ -19,5 +19,6 @@
 package org.apache.fineract.interoperation.domain;
 
 public enum InteropActionState {
-    ACCEPTED, REJECTED
+    ACCEPTED, //
+    REJECTED; //
 }
