@@ -127,8 +127,8 @@ public class LoanProductTestBuilder {
     private String interestRecalculationCompoundingMethod = "0";
     private String preCloseInterestCalculationStrategy = INTEREST_APPLICABLE_STRATEGY_ON_PRE_CLOSE_DATE;
     private String rescheduleStrategyMethod = "1";
-    private String recalculationRestFrequencyType = "1";
-    private String recalculationRestFrequencyInterval = "0";
+    private String recalculationRestFrequencyType = "2";
+    private String recalculationRestFrequencyInterval = "1";
     private String recalculationCompoundingFrequencyType = null;
     private String recalculationCompoundingFrequencyInterval = null;
     private String minimumDaysBetweenDisbursalAndFirstRepayment = null;
@@ -166,6 +166,8 @@ public class LoanProductTestBuilder {
     private List<String> supportedInterestRefundTypes = null;
     private String chargeOffBehaviour;
     private boolean interestRecognitionOnDisbursementDate = false;
+    private Boolean enableBuyDownFee = false;
+    private String buyDownFeeCalculationType;
 
     public String build() {
         final HashMap<String, Object> map = build(null, null);
@@ -341,6 +343,14 @@ public class LoanProductTestBuilder {
             map.put("chargeOffBehaviour", chargeOffBehaviour);
         }
 
+        if (this.enableBuyDownFee != null) {
+            map.put("enableBuyDownFee", this.enableBuyDownFee);
+        }
+
+        if (this.buyDownFeeCalculationType != null) {
+            map.put("buyDownFeeCalculationType", this.buyDownFeeCalculationType);
+        }
+
         return map;
     }
 
@@ -906,4 +916,13 @@ public class LoanProductTestBuilder {
         }
     }
 
+    public LoanProductTestBuilder withEnableBuyDownFee(final Boolean enableBuyDownFee) {
+        this.enableBuyDownFee = enableBuyDownFee;
+        return this;
+    }
+
+    public LoanProductTestBuilder withBuyDownFeeCalculationType(final String buyDownFeeCalculationType) {
+        this.buyDownFeeCalculationType = buyDownFeeCalculationType;
+        return this;
+    }
 }
